@@ -10,6 +10,18 @@ you will screw up the layout where they are used in case expressions!
 
 #endif
 
+/* Useful in the headers that we share with the RTS */
+#define COMPILING_GHC 1
+
+/* Pull in all the platform defines for this build (foo_TARGET_ARCH etc.) */
+#include "ghc_boot_platform.h"
+
+/* Pull in the autoconf defines (HAVE_FOO), but don't include
+ * ghcconfig.h, because that will include ghcplatform.h which has the
+ * wrong platform settings for the compiler (it has the platform
+ * settings for the target plat instead). */
+#include "../includes/ghcautoconf.h"
+
 #if __GLASGOW_HASKELL__ >= 504
 
 #define CONCURRENT  Control.Concurrent
@@ -52,27 +64,26 @@ name = Util.global (value) :: IORef (ty); \
 {-# NOINLINE name #-}
 #endif
 
+#if __GLASGOW_HASKELL__ >= 620
+#define UNBOX_FIELD !!
+#else
+#define UNBOX_FIELD !
+#endif
+
 #define COMMA ,
 
 #ifdef DEBUG
 #define ASSERT(e) if (not (e)) then (assertPanic __FILE__ __LINE__) else
 #define ASSERT2(e,msg) if (not (e)) then (assertPprPanic __FILE__ __LINE__ (msg)) else
 #define WARN( e, msg ) (warnPprTrace (e) __FILE__ __LINE__ (msg))
-#define ASSERTM(e) ASSERT(e) do
+#define ASSERTM(mbool) do { bool <- mbool; ASSERT(bool) return () }
+#define ASSERTM2(mbool,msg) do { bool <- mbool; ASSERT2(bool,msg) return () }
 #else
-#define ASSERT(e)
-#define ASSERT2(e,msg)
+#define ASSERT(e)      if False then error "ASSERT"  else
+#define ASSERT2(e,msg) if False then error "ASSERT2" else
 #define ASSERTM(e)
-#define WARN(e,msg)
-#endif
-
--- temporary usage assertion control KSW 2000-10
-#ifdef DO_USAGES
-#define UASSERT(e) ASSERT(e)
-#define UASSERT2(e,msg) ASSERT2(e,msg)
-#else
-#define UASSERT(e)
-#define UASSERT2(e,msg)
+#define ASSERTM2(e,msg)
+#define WARN(e,msg)    if False then error "WARN"    else
 #endif
 
 -- This #ifndef lets us switch off the "import FastString"
