@@ -1,6 +1,6 @@
 {-# OPTIONS -#include "Linker.h" #-}
 -----------------------------------------------------------------------------
--- $Id: InteractiveUI.hs,v 1.186.2.10 2005/07/06 10:44:44 simonmar Exp $
+-- $Id: InteractiveUI.hs,v 1.186.2.12 2006/03/23 14:19:25 simonmar Exp $
 --
 -- GHC Interactive User Interface
 --
@@ -137,6 +137,7 @@ helpText =
  "   :set <option> ...           set options\n" ++
  "   :set args <arg> ...         set the arguments returned by System.getArgs\n" ++
  "   :set prog <progname>        set the value returned by System.getProgName\n" ++
+ "   :set prompt <prompt>        set the prompt used in GHCi\n" ++
  "\n" ++
  "   :show modules               show the currently loaded modules\n" ++
  "   :show bindings              show the current bindings made at the prompt\n" ++
@@ -191,16 +192,10 @@ interactiveUI dflags srcs maybe_expr = do
    Readline.initialize
 #endif
 
-#if defined(mingw32_HOST_OS)
-    -- The win32 Console API mutates the first character of 
-    -- type-ahead when reading from it in a non-buffered manner. Work
-    -- around this by flushing the input buffer of type-ahead characters.
-    -- 
-   GHC.ConsoleHandler.flushConsole stdin
-#endif
    startGHCi (runGHCi srcs dflags maybe_expr)
 	GHCiState{ progname = "<interactive>",
 		   args = [],
+                   prompt = "%s> ",
 		   targets = srcs,
 		   cmstate = cmstate,
 		   options = [] }
@@ -260,6 +255,18 @@ runGHCi paths dflags maybe_expr = do
 
   case maybe_expr of
 	Nothing -> 
+#if defined(mingw32_HOST_OS)
+          do
+            -- The win32 Console API mutates the first character of 
+            -- type-ahead when reading from it in a non-buffered manner. Work
+            -- around this by flushing the input buffer of type-ahead characters,
+            -- but only if stdin is available.
+            flushed <- io (IO.try (GHC.ConsoleHandler.flushConsole stdin))
+            case flushed of 
+   	     Left err | isDoesNotExistError err -> return ()
+   		      | otherwise -> io (ioError err)
+   	     Right () -> return ()
+#endif
 	    -- enter the interactive loop
 	    interactiveLoop is_tty show_prompt
 	Just expr -> do
@@ -323,10 +330,11 @@ checkPerms name =
 #endif
 
 fileLoop :: Handle -> Bool -> GHCi ()
-fileLoop hdl prompt = do
+fileLoop hdl show_prompt = do
    cmstate <- getCmState
    (mod,imports) <- io (cmGetContext cmstate)
-   when prompt (io (putStr (mkPrompt mod imports)))
+   st <- getGHCiState
+   when show_prompt (io (putStr (mkPrompt mod imports (prompt st))))
    l <- io (IO.try (hGetLine hdl))
    case l of
 	Left e | isEOFError e		   -> return ()
@@ -339,9 +347,9 @@ fileLoop hdl prompt = do
 		-- EOF.
 	Right l -> 
 	  case remove_spaces l of
-	    "" -> fileLoop hdl prompt
+	    "" -> fileLoop hdl show_prompt
 	    l  -> do quit <- runCommand l
-          	     if quit then return () else fileLoop hdl prompt
+          	     if quit then return () else fileLoop hdl show_prompt
 
 stringLoop :: [String] -> GHCi ()
 stringLoop [] = return ()
@@ -351,8 +359,17 @@ stringLoop (s:ss) = do
 	l  -> do quit <- runCommand l
                  if quit then return () else stringLoop ss
 
-mkPrompt toplevs exports
-   = concat (intersperse " " (map ('*':) toplevs ++ exports)) ++ "> "
+mkPrompt toplevs exports prompt
+  = showSDoc $ f prompt
+    where
+        f ('%':'s':xs) = perc_s <> f xs
+        f ('%':'%':xs) = char '%' <> f xs
+        f (x:xs) = char x <> f xs
+        f [] = empty
+    
+        perc_s = hsep (map (\m -> char '*' <> text m) toplevs) <+>
+                 hsep (map text exports)
+             
 
 #ifdef USE_READLINE
 readlineLoop :: GHCi ()
@@ -360,7 +377,8 @@ readlineLoop = do
    cmstate <- getCmState
    (mod,imports) <- io (cmGetContext cmstate)
    io yield
-   l <- io (readline (mkPrompt mod imports)
+   st <- getGHCiState
+   l <- io (readline (mkPrompt mod imports (prompt st))
 	  	`finally` setNonBlockingFD 0)
 		-- readline sometimes puts stdin into blocking mode,
 		-- so we need to put it back for the IO library
@@ -934,6 +952,7 @@ setCmd str
   = case words str of
 	("args":args) -> setArgs args
 	("prog":prog) -> setProg prog
+        ("prompt":prompt) -> setPrompt (dropWhile isSpace $ drop 6 $ dropWhile isSpace str)
 	wds -> setOptions wds
 
 setArgs args = do
@@ -946,6 +965,15 @@ setProg [prog] = do
 setProg _ = do
   io (hPutStrLn stderr "syntax: :set prog <progname>")
 
+setPrompt value = do
+  st <- getGHCiState
+  if null value
+      then io $ hPutStrLn stderr $ "syntax: :set prompt <prompt>, currently \"" ++ prompt st ++ "\""
+      else setGHCiState st{ prompt = remQuotes value }
+  where
+     remQuotes ('\"':xs) | not (null xs) && last xs == '\"' = init xs
+     remQuotes x = x
+
 setOptions wds =
    do -- first, deal with the GHCi opts (+s, +t, etc.)
       let (plus_opts, minus_opts)  = partition isPlus wds
@@ -1058,6 +1086,7 @@ data GHCiState = GHCiState
      { 
 	progname       :: String,
 	args	       :: [String],
+        prompt         :: String,
 	targets        :: [FilePath],
 	cmstate        :: CmState,
 	options        :: [GHCiOption]
