@@ -1,5 +1,5 @@
 -----------------------------------------------------------------------------
--- $Id: DriverMkDepend.hs,v 1.39.2.3 2005/02/23 14:22:38 simonmar Exp $
+-- $Id: DriverMkDepend.hs,v 1.39.2.5 2005/07/28 16:28:10 simonpj Exp $
 --
 -- GHC Driver
 --
@@ -54,7 +54,8 @@ doMkDependHS dflags srcs
 
 		-- Do the downsweep to find all the modules
 	; excl_mods <- readIORef v_Dep_exclude_mods
-	; mod_summaries <- cmDownsweep dflags srcs [] excl_mods
+	; mod_summaries <- cmDownsweep dflags srcs [] excl_mods 
+					True {- Allow dup roots -}
 
 		-- Sort into dependency order
 		-- There should be no cycles
@@ -67,7 +68,7 @@ doMkDependHS dflags srcs
 		
 		-- Prcess them one by one, dumping results into makefile
 		-- and complaining about cycles
-	; mapM (processDeps dflags (mkd_tmp_hdl files)) sorted
+	; mapM (processDeps dflags excl_mods (mkd_tmp_hdl files)) sorted
 
 		-- Tidy up
 	; endMkDependHS dflags files }
@@ -146,6 +147,7 @@ beginMkDependHS = do
 -----------------------------------------------------------------
 
 processDeps :: DynFlags
+	    -> [Module]
 	    -> Handle		-- Write dependencies to here
 	    -> SCC ModSummary
 	    -> IO ()
@@ -164,11 +166,11 @@ processDeps :: DynFlags
 --
 -- For {-# SOURCE #-} imports the "hi" will be "hi-boot".
 
-processDeps dflags hdl (CyclicSCC nodes)
+processDeps dflags excl_mods hdl (CyclicSCC nodes)
   =	-- There shouldn't be any cycles; report them	
     throwDyn (ProgramError (showSDoc $ cyclicModuleErr nodes))
 
-processDeps dflags hdl (AcyclicSCC node)
+processDeps dflags excl_mods hdl (AcyclicSCC node)
   = do	{ extra_suffixes   <- readIORef v_Dep_suffixes
 	; include_pkg_deps <- readIORef v_Dep_include_pkg_deps
 	; let src_file  = msHsFilePath node
@@ -195,8 +197,13 @@ processDeps dflags hdl (AcyclicSCC node)
 	; writeDependency hdl obj_files src_file
 
 		-- Emit a dependency for each import
-	; mapM_ (do_imp True)  (ms_srcimps node)	-- SOURCE imports
-	; mapM_ (do_imp False) (ms_imps node)		-- regular imports
+	-- SOURCE imports
+	; mapM_ (do_imp True)  
+		(filter (`notElem` excl_mods) (ms_srcimps node))
+
+	-- regular imports
+	; mapM_ (do_imp False)
+		(filter (`notElem` excl_mods) (ms_imps node))
 	}
 
 
