@@ -193,6 +193,7 @@ compile hsc_env mod_summary
    let do_recomp = recompFlag dyn_flags
        source_unchanged' = source_unchanged && do_recomp
        hsc_env' = hsc_env { hsc_dflags = dyn_flags' }
+       object_filename = ml_obj_file location
 
    -- run the compiler
    hsc_result <- hscMain hsc_env' printErrorsAndWarnings mod_summary
@@ -211,11 +212,12 @@ compile hsc_env mod_summary
 
 	| otherwise		-- Normal Haskell source files
 	-> do
-	   let 
-	   maybe_stub_o <- compileStub dyn_flags' stub_c_exists
-	   let stub_unlinked = case maybe_stub_o of
-				  Nothing -> []
-				  Just stub_o -> [ DotO stub_o ]
+	   stub_unlinked <-
+	     if stub_c_exists then do
+		stub_o <- compileStub dyn_flags' object_filename
+		return [ DotO stub_o ]
+	     else
+		return []
 
 	   (hs_unlinked, unlinked_time) <-
 	     case hsc_lang of
@@ -254,17 +256,33 @@ compile hsc_env mod_summary
 -----------------------------------------------------------------------------
 -- stub .h and .c files (for foreign export support)
 
-compileStub dflags stub_c_exists
-  | not stub_c_exists = return Nothing
-  | stub_c_exists = do
+-- The _stub.c file is derived from the haskell source file (but stored
+-- in hscStubCOutName in the dflags for some reason, probably historical).
+-- Consequently, we derive the _stub.o filename from the haskell object
+-- filename.  
+--
+-- This isn't necessarily the same as the object filename we
+-- would get if we just compiled the _stub.c file using the pipeline.
+-- For example:
+--
+--    ghc src/A.hs -odir obj
+-- 
+-- results in obj/A.o, and src/A_stub.c.  If we compile src/A_stub.c with
+-- -odir obj, we would get obj/src/A_stub.o, which is wrong; we want
+-- obj/A_stub.o.
+
+compileStub dflags object_filename = do
+	let (o_base, o_ext) = splitFilename object_filename
+	    stub_o = o_base ++ "_stub." ++ o_ext
+
 	-- compile the _stub.c file w/ gcc
 	let stub_c = hscStubCOutName dflags
-	(_, stub_o) <- runPipeline StopLn "stub-compile" dflags
+	runPipeline StopLn "stub-compile" dflags
 			    True{-persistent output-} 
-			    Nothing{-no specific output file-}
+			    (Just stub_o)
 			    stub_c
 			    Nothing{-no ModLocation-}
-	return (Just stub_o)
+	return stub_o
 
 
 -- ---------------------------------------------------------------------------
@@ -313,14 +331,34 @@ link Batch dflags batch_attempt_linking hpt
 	          return Succeeded
 	  else do
 
-	when (verb >= 1) $
-             hPutStrLn stderr "Linking ..."
-
+        o_file <- readIORef v_Output_file
 	let getOfiles (LM _ _ us) = map nameOfObject (filter isObject us)
 	    obj_files = concatMap getOfiles linkables
 
+	    exe_file = exeFileName o_file
+
+	-- if the modification time on the executable is later than the
+	-- modification times on all of the objects, then omit linking
+	-- (unless the -no-recomp flag was given).
+	e_exe_time <- IO.try $ getModificationTime exe_file
+	let linking_needed 
+		| Left _  <- e_exe_time = True
+		| Right t <- e_exe_time = 
+			any (t <) (map linkableTime linkables)
+
+	if recompFlag dflags && not linking_needed
+	   then do compilationProgressMsg dflags (exe_file ++ " is up to date, linking not required.")
+		   return Succeeded
+	   else do
+
+	compilationProgressMsg dflags "Linking ..."
+
 	-- Don't showPass in Batch mode; doLink will do that for us.
-        staticLink dflags obj_files pkg_deps
+	ghc_link <- readIORef v_GhcLink
+	let link = case ghc_link of
+	        MkDLL       -> doMkDLL
+	        StaticLink  -> staticLink
+	link dflags obj_files pkg_deps
 
         when (verb >= 3) (hPutStrLn stderr "link: done")
 
@@ -696,11 +734,9 @@ runPhase (Hsc src_flavour) stop dflags basename suff input_fn get_output_fn _may
 		      stub_h_exists stub_c_exists
 		      _maybe_interpreted_code -> do
 
-		-- Deal with stubs 
-		maybe_stub_o <- compileStub dflags' stub_c_exists
-		case maybe_stub_o of
-		      Nothing     -> return ()
-		      Just stub_o -> add v_Ld_inputs stub_o
+		when stub_c_exists $ do
+			stub_o <- compileStub dflags' o_file
+			add v_Ld_inputs stub_o
 
 		-- In the case of hs-boot files, generate a dummy .o-boot 
 		-- stamp file for the benefit of Make
@@ -780,11 +816,26 @@ runPhase cc_phase stop dflags basename suff input_fn get_output_fn maybe_loc
 		| otherwise         = As
 	output_fn <- get_output_fn next_phase maybe_loc
 
+	let
+	  more_hcc_opts =
+#if i386_TARGET_ARCH
+	   	-- on x86 the floating point regs have greater precision
+	     	-- than a double, which leads to unpredictable results.
+		-- By default, we turn this off with -ffloat-store unless
+		-- the user specified -fexcess-precision.
+		(if excessPrecision then [] else [ "-ffloat-store" ]) ++
+#endif
+		-- gcc's -fstrict-aliasing allows two accesses to memory
+		-- to be considered non-aliasing if they have different types.
+		-- This interacts badly with the C code we generate, which is
+		-- very weakly typed, being derived from C--.
+		["-fno-strict-aliasing"]
+
 	-- force the C compiler to interpret this file as C when
 	-- compiling .hc files, by adding the -x c option.
 	let langopt | hcc = [ SysTools.Option "-x", SysTools.Option "c"]
 		    | otherwise = [ ]
-
+  
 	SysTools.runCc dflags (langopt ++
 			[ SysTools.FileOption "" input_fn
 			, SysTools.Option "-o"
@@ -796,11 +847,13 @@ runPhase cc_phase stop dflags basename suff input_fn get_output_fn maybe_loc
 		       ++ (if hcc && mangle
 		  	     then md_regd_c_flags
 		  	     else [])
+		       ++ (if hcc 
+			     then more_hcc_opts
+			     else [])
 		       ++ [ verb, "-S", "-Wimplicit", "-O" ]
 		       ++ [ "-D__GLASGOW_HASKELL__="++cProjectVersionInt ]
 		       ++ cc_opts
 		       ++ split_opt
-		       ++ (if excessPrecision then [] else [ "-ffloat-store" ])
 		       ++ include_paths
 		       ++ pkg_extra_cc_opts
 		       ))
@@ -1069,6 +1122,8 @@ getHCFilePackages filename =
 staticLink :: DynFlags -> [FilePath] -> [PackageId] -> IO ()
 staticLink dflags o_files dep_packages = do
     let verb = getVerbFlag dflags
+    o_file <- readIORef v_Output_file
+    let  output_fn = exeFileName o_file
     static     <- readIORef v_Static
     no_hs_main <- readIORef v_NoHsMain
 
@@ -1076,13 +1131,6 @@ staticLink dflags o_files dep_packages = do
     -- explicit packages with the auto packages and all of their
     -- dependencies, and eliminating duplicates.
 
-    o_file <- readIORef v_Output_file
-#if defined(mingw32_HOST_OS)
-    let output_fn = case o_file of { Just s -> s; Nothing -> "main.exe"; }
-#else
-    let output_fn = case o_file of { Just s -> s; Nothing -> "a.out"; }
-#endif
-
     pkg_lib_paths <- getPackageLibraryPath dflags dep_packages
     let pkg_lib_path_opts = map ("-L"++) pkg_lib_paths
 
@@ -1168,6 +1216,17 @@ staticLink dflags o_files dep_packages = do
              if success then return ()
                         else throwDyn (InstallationError ("cannot move binary to PVM dir")))
 
+
+exeFileName :: Maybe FilePath -> FilePath
+exeFileName o_file
+  | Just s <- o_file = s
+  | otherwise = 
+#if defined(mingw32_HOST_OS)
+	"main.exe"
+#else
+	"a.out"
+#endif
+
 -----------------------------------------------------------------------------
 -- Making a DLL (only for Win32)
 
