@@ -1,7 +1,7 @@
 {-# OPTIONS -fno-warn-incomplete-patterns -optc-DNON_POSIX_SOURCE #-}
 
 -----------------------------------------------------------------------------
--- $Id: Main.hs,v 1.147.2.1 2005/02/10 15:30:43 simonmar Exp $
+-- $Id: Main.hs,v 1.147.2.3 2005/12/02 12:52:15 simonmar Exp $
 --
 -- GHC Driver program
 --
@@ -47,6 +47,7 @@ import Util
 import Panic		( GhcException(..), panic, installSignalHandlers )
 
 import DATA_IOREF	( readIORef, writeIORef )
+import DYNAMIC		( fromDynamic )
 import EXCEPTION	( throwDyn, Exception(..), 
 			  AsyncException(StackOverflow) )
 
@@ -82,26 +83,24 @@ import Maybe
 
 main =
   -- top-level exception handler: any unrecognised exception is a compiler bug.
-  handle (\exception -> do
+  let handler exception = 
+	 handle handler $ do
   	   hFlush stdout
 	   case exception of
 		-- an IO exception probably isn't our fault, so don't panic
 		IOException _ ->  hPutStrLn stderr (show exception)
 		AsyncException StackOverflow ->
 			hPutStrLn stderr "stack overflow: use +RTS -K<size> to increase it"
+		DynException e | Just dyn <- fromDynamic e ->
+  			case dyn of
+			     PhaseFailed _ code -> exitWith code
+			     Interrupted -> exitWith (ExitFailure 1)
+			     _ -> do hPutStrLn stderr (show (dyn :: GhcException))
+				     exitWith (ExitFailure 1)
 		_other ->  hPutStr stderr (show (Panic (show exception)))
 	   exitWith (ExitFailure 1)
-         ) $ do
-
-  -- all error messages are propagated as exceptions
-  handleDyn (\dyn -> do
-  		hFlush stdout
-  		case dyn of
-		     PhaseFailed _ code -> exitWith code
-		     Interrupted -> exitWith (ExitFailure 1)
-		     _ -> do hPutStrLn stderr (show (dyn :: GhcException))
-			     exitWith (ExitFailure 1)
-	    ) $ do
+   in
+   handle handler $ do
 
    installSignalHandlers
 
@@ -326,4 +325,9 @@ showBanner mode dflags = do
 	do hPutStr stderr "Glasgow Haskell Compiler, Version "
  	   hPutStr stderr cProjectVersion
 	   hPutStr stderr ", for Haskell 98, compiled by GHC version "
+#ifdef GHCI
+	   -- GHCI is only set when we are bootstrapping...
+ 	   hPutStrLn stderr cProjectVersion
+#else
 	   hPutStrLn stderr cBooterVersion
+#endif
