@@ -13,14 +13,18 @@ import SrcLoc
 import Outputable
 import Panic     ( GhcException(..) )
 import EXCEPTION ( throwDyn )
+#if __GLASGOW_HASKELL__ >= 503
+import Data.Array
+#else
 import Array
+#endif
 #if __GLASGOW_HASKELL__ >= 503
 import GHC.Exts
 #else
 import GlaExts
 #endif
 
--- parser produced by Happy Version 1.15
+-- parser produced by Happy Version 1.16
 
 newtype HappyAbsSyn  = HappyAbsSyn (() -> ())
 happyIn4 :: ([ PackageConfig ]) -> (HappyAbsSyn )
@@ -155,14 +159,14 @@ happyReduceArr = array (1, 26) [
 happy_n_terms = 12 :: Int
 happy_n_nonterms = 13 :: Int
 
-happyReduce_1 = happySpecReduce_2 0# happyReduction_1
+happyReduce_1 = happySpecReduce_2  0# happyReduction_1
 happyReduction_1 happy_x_2
 	happy_x_1
 	 =  happyIn4
 		 ([]
 	)
 
-happyReduce_2 = happySpecReduce_3 0# happyReduction_2
+happyReduce_2 = happySpecReduce_3  0# happyReduction_2
 happyReduction_2 happy_x_3
 	happy_x_2
 	happy_x_1
@@ -171,14 +175,14 @@ happyReduction_2 happy_x_3
 		 (reverse happy_var_2
 	)}
 
-happyReduce_3 = happySpecReduce_1 1# happyReduction_3
+happyReduce_3 = happySpecReduce_1  1# happyReduction_3
 happyReduction_3 happy_x_1
 	 =  case happyOut6 happy_x_1 of { happy_var_1 -> 
 	happyIn5
 		 ([ happy_var_1 ]
 	)}
 
-happyReduce_4 = happySpecReduce_3 1# happyReduction_4
+happyReduce_4 = happySpecReduce_3  1# happyReduction_4
 happyReduction_4 happy_x_3
 	happy_x_2
 	happy_x_1
@@ -199,14 +203,14 @@ happyReduction_5 (happy_x_4 `HappyStk`
 		 (happy_var_3 defaultPackageConfig
 	) `HappyStk` happyRest}
 
-happyReduce_6 = happySpecReduce_1 3# happyReduction_6
+happyReduce_6 = happySpecReduce_1  3# happyReduction_6
 happyReduction_6 happy_x_1
 	 =  case happyOut8 happy_x_1 of { happy_var_1 -> 
 	happyIn7
 		 (\p -> happy_var_1 p
 	)}
 
-happyReduce_7 = happySpecReduce_3 3# happyReduction_7
+happyReduce_7 = happySpecReduce_3  3# happyReduction_7
 happyReduction_7 happy_x_3
 	happy_x_2
 	happy_x_1
@@ -220,15 +224,15 @@ happyReduce_8 = happyMonadReduce 3# 4# happyReduction_8
 happyReduction_8 (happy_x_3 `HappyStk`
 	happy_x_2 `HappyStk`
 	happy_x_1 `HappyStk`
-	happyRest)
+	happyRest) tk
 	 = happyThen (case happyOutTok happy_x_1 of { (L _ (ITvarid    happy_var_1)) -> 
 	case happyOut9 happy_x_3 of { happy_var_3 -> 
-	 case unpackFS happy_var_1 of
+	( case unpackFS happy_var_1 of
 		        "package"     -> return (\p -> p{package = happy_var_3})
-			_other        -> happyError}}
+			_other        -> happyError)}}
 	) (\r -> happyReturn (happyIn8 r))
 
-happyReduce_9 = happySpecReduce_3 4# happyReduction_9
+happyReduce_9 = happySpecReduce_3  4# happyReduction_9
 happyReduction_9 happy_x_3
 	happy_x_2
 	happy_x_1
@@ -240,17 +244,17 @@ happyReduce_10 = happyMonadReduce 3# 4# happyReduction_10
 happyReduction_10 (happy_x_3 `HappyStk`
 	happy_x_2 `HappyStk`
 	happy_x_1 `HappyStk`
-	happyRest)
+	happyRest) tk
 	 = happyThen (case happyOutTok happy_x_1 of { (L _ (ITvarid    happy_var_1)) -> 
 	case happyOutTok happy_x_3 of { (L _ (ITconid    happy_var_3)) -> 
-	 case unpackFS happy_var_1 of {
+	( case unpackFS happy_var_1 of {
 		   	"exposed" -> 
 			   case unpackFS happy_var_3 of {
 				"True"  -> return (\p -> p{exposed=True});
 				"False" -> return (\p -> p{exposed=False});
 				_       -> happyError };
 		   	"license" -> return id; -- not interested
-		   	_         -> happyError }}}
+		   	_         -> happyError })}}
 	) (\r -> happyReturn (happyIn8 r))
 
 happyReduce_11 = happyReduce 4# 4# happyReduction_11
@@ -263,7 +267,7 @@ happyReduction_11 (happy_x_4 `HappyStk`
 		 (id
 	) `HappyStk` happyRest
 
-happyReduce_12 = happySpecReduce_3 4# happyReduction_12
+happyReduce_12 = happySpecReduce_3  4# happyReduction_12
 happyReduction_12 happy_x_3
 	happy_x_2
 	happy_x_1
@@ -277,6 +281,7 @@ happyReduction_12 happy_x_3
 		        "libraryDirs"       -> p{libraryDirs       = happy_var_3}
 		        "hsLibraries"       -> p{hsLibraries       = happy_var_3}
 		        "extraLibraries"    -> p{extraLibraries    = happy_var_3}
+		        "extraGHCiLibraries"-> p{extraGHCiLibraries= happy_var_3}
 		        "includeDirs"       -> p{includeDirs       = happy_var_3}
 		        "includes"          -> p{includes          = happy_var_3}
 		        "hugsOptions"       -> p{hugsOptions       = happy_var_3}
@@ -295,12 +300,12 @@ happyReduce_13 = happyMonadReduce 3# 4# happyReduction_13
 happyReduction_13 (happy_x_3 `HappyStk`
 	happy_x_2 `HappyStk`
 	happy_x_1 `HappyStk`
-	happyRest)
+	happyRest) tk
 	 = happyThen (case happyOutTok happy_x_1 of { (L _ (ITvarid    happy_var_1)) -> 
 	case happyOut11 happy_x_3 of { happy_var_3 -> 
-	 case unpackFS happy_var_1 of
+	( case unpackFS happy_var_1 of
 		        "depends"     -> return (\p -> p{depends = happy_var_3})
-			_other        -> happyError}}
+			_other        -> happyError)}}
 	) (\r -> happyReturn (happyIn8 r))
 
 happyReduce_14 = happyReduce 10# 5# happyReduction_14
@@ -340,7 +345,7 @@ happyReduction_15 (happy_x_10 `HappyStk`
 		 (Version{ versionBranch=happy_var_5, versionTags=happy_var_9 }
 	) `HappyStk` happyRest}}
 
-happyReduce_16 = happySpecReduce_3 7# happyReduction_16
+happyReduce_16 = happySpecReduce_3  7# happyReduction_16
 happyReduction_16 happy_x_3
 	happy_x_2
 	happy_x_1
@@ -349,14 +354,14 @@ happyReduction_16 happy_x_3
 		 (happy_var_2
 	)}
 
-happyReduce_17 = happySpecReduce_1 8# happyReduction_17
+happyReduce_17 = happySpecReduce_1  8# happyReduction_17
 happyReduction_17 happy_x_1
 	 =  case happyOut9 happy_x_1 of { happy_var_1 -> 
 	happyIn12
 		 ([ happy_var_1 ]
 	)}
 
-happyReduce_18 = happySpecReduce_3 8# happyReduction_18
+happyReduce_18 = happySpecReduce_3  8# happyReduction_18
 happyReduction_18 happy_x_3
 	happy_x_2
 	happy_x_1
@@ -366,14 +371,14 @@ happyReduction_18 happy_x_3
 		 (happy_var_1 : happy_var_3
 	)}}
 
-happyReduce_19 = happySpecReduce_2 9# happyReduction_19
+happyReduce_19 = happySpecReduce_2  9# happyReduction_19
 happyReduction_19 happy_x_2
 	happy_x_1
 	 =  happyIn13
 		 ([]
 	)
 
-happyReduce_20 = happySpecReduce_3 9# happyReduction_20
+happyReduce_20 = happySpecReduce_3  9# happyReduction_20
 happyReduction_20 happy_x_3
 	happy_x_2
 	happy_x_1
@@ -382,14 +387,14 @@ happyReduction_20 happy_x_3
 		 (happy_var_2
 	)}
 
-happyReduce_21 = happySpecReduce_1 10# happyReduction_21
+happyReduce_21 = happySpecReduce_1  10# happyReduction_21
 happyReduction_21 happy_x_1
 	 =  case happyOutTok happy_x_1 of { (L _ (ITinteger  happy_var_1)) -> 
 	happyIn14
 		 ([ fromIntegral happy_var_1 ]
 	)}
 
-happyReduce_22 = happySpecReduce_3 10# happyReduction_22
+happyReduce_22 = happySpecReduce_3  10# happyReduction_22
 happyReduction_22 happy_x_3
 	happy_x_2
 	happy_x_1
@@ -399,14 +404,14 @@ happyReduction_22 happy_x_3
 		 (fromIntegral happy_var_1 : happy_var_3
 	)}}
 
-happyReduce_23 = happySpecReduce_2 11# happyReduction_23
+happyReduce_23 = happySpecReduce_2  11# happyReduction_23
 happyReduction_23 happy_x_2
 	happy_x_1
 	 =  happyIn15
 		 ([]
 	)
 
-happyReduce_24 = happySpecReduce_3 11# happyReduction_24
+happyReduce_24 = happySpecReduce_3  11# happyReduction_24
 happyReduction_24 happy_x_3
 	happy_x_2
 	happy_x_1
@@ -415,14 +420,14 @@ happyReduction_24 happy_x_3
 		 (happy_var_2
 	)}
 
-happyReduce_25 = happySpecReduce_1 12# happyReduction_25
+happyReduce_25 = happySpecReduce_1  12# happyReduction_25
 happyReduction_25 happy_x_1
 	 =  case happyOutTok happy_x_1 of { (L _ (ITstring   happy_var_1)) -> 
 	happyIn16
 		 ([ unpackFS happy_var_1 ]
 	)}
 
-happyReduce_26 = happySpecReduce_3 12# happyReduction_26
+happyReduce_26 = happySpecReduce_3  12# happyReduction_26
 happyReduction_26 happy_x_3
 	happy_x_2
 	happy_x_1
@@ -436,7 +441,7 @@ happyNewToken action sts stk
 	= lexer(\tk -> 
 	let cont i = happyDoAction i tk action sts stk in
 	case tk of {
-	L _ ITeof -> happyDoAction 11# (error "reading EOF!") action sts stk;
+	L _ ITeof -> happyDoAction 11# tk action sts stk;
 	L _ ITocurly -> cont 1#;
 	L _ ITccurly -> cont 2#;
 	L _ ITobrack -> cont 3#;
@@ -447,10 +452,10 @@ happyNewToken action sts stk
 	L _ (ITconid    happy_dollar_dollar) -> cont 8#;
 	L _ (ITstring   happy_dollar_dollar) -> cont 9#;
 	L _ (ITinteger  happy_dollar_dollar) -> cont 10#;
-	_ -> happyError'
+	_ -> happyError' tk
 	})
 
-happyError_ tk = happyError'
+happyError_ tk = happyError' tk
 
 happyThen :: () => P a -> (a -> P b) -> P b
 happyThen = (>>=)
@@ -459,8 +464,8 @@ happyReturn = (return)
 happyThen1 = happyThen
 happyReturn1 :: () => a -> P a
 happyReturn1 = happyReturn
-happyError' :: () => P a
-happyError' = happyError
+happyError' :: () => Located Token -> P a
+happyError' tk = (\token -> happyError) tk
 
 parse = happySomeParser where
   happySomeParser = happyThen (happyParse 0#) (\x -> happyReturn (happyOut4 x))
@@ -481,22 +486,12 @@ loadPackageConfig conf_filename = do
 	POk _ pkg_details -> do
 	    return pkg_details
 {-# LINE 1 "GenericTemplate.hs" #-}
--- $Id$
-
-
-
-
-
-
-
-
-
-
-
-
-
-{-# LINE 27 "GenericTemplate.hs" #-}
+{-# LINE 1 "<built-in>" #-}
+{-# LINE 1 "<command line>" #-}
+{-# LINE 1 "GenericTemplate.hs" #-}
+-- Id: GenericTemplate.hs,v 1.26 2005/01/14 14:47:22 simonmar Exp 
 
+{-# LINE 28 "GenericTemplate.hs" #-}
 
 
 data Happy_IntList = HappyCons Int# Happy_IntList
@@ -505,38 +500,11 @@ data Happy_IntList = HappyCons Int# Happy_IntList
 
 
 
+{-# LINE 49 "GenericTemplate.hs" #-}
 
+{-# LINE 59 "GenericTemplate.hs" #-}
 
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
+{-# LINE 68 "GenericTemplate.hs" #-}
 
 infixr 9 `HappyStk`
 data HappyStk a = HappyStk a (HappyStk a)
@@ -588,14 +556,7 @@ happyDoAction i tk st
  	 action | check     = indexShortOffAddr happyTable off_i
 		| otherwise = indexShortOffAddr happyDefActions st
 
-
-
-
-
-
-
-
-
+{-# LINE 127 "GenericTemplate.hs" #-}
 
 
 indexShortOffAddr (HappyA# arr) off =
@@ -628,8 +589,7 @@ data HappyAddr = HappyA# Addr#
 -----------------------------------------------------------------------------
 -- HappyState data type (not arrays)
 
-{-# LINE 169 "GenericTemplate.hs" #-}
-
+{-# LINE 170 "GenericTemplate.hs" #-}
 
 -----------------------------------------------------------------------------
 -- Shifting a token
@@ -678,10 +638,24 @@ happyReduce k nt fn j tk st sts stk
 happyMonadReduce k nt fn 0# tk st sts stk
      = happyFail 0# tk st sts stk
 happyMonadReduce k nt fn j tk st sts stk =
-        happyThen1 (fn stk) (\r -> happyGoto nt j tk st1 sts1 (r `HappyStk` drop_stk))
+        happyThen1 (fn stk tk) (\r -> happyGoto nt j tk st1 sts1 (r `HappyStk` drop_stk))
        where sts1@((HappyCons (st1@(action)) (_))) = happyDrop k (HappyCons (st) (sts))
              drop_stk = happyDropStk k stk
 
+happyMonad2Reduce k nt fn 0# tk st sts stk
+     = happyFail 0# tk st sts stk
+happyMonad2Reduce k nt fn j tk st sts stk =
+       happyThen1 (fn stk tk) (\r -> happyNewToken new_state sts1 (r `HappyStk` drop_stk))
+       where sts1@((HappyCons (st1@(action)) (_))) = happyDrop k (HappyCons (st) (sts))
+             drop_stk = happyDropStk k stk
+
+             off    = indexShortOffAddr happyGotoOffsets st1
+             off_i  = (off +# nt)
+             new_state = indexShortOffAddr happyTable off_i
+
+
+
+
 happyDrop 0# l = l
 happyDrop n (HappyCons (_) (t)) = happyDrop (n -# (1# :: Int#)) t
 
