@@ -22,7 +22,6 @@
 #include "OSThreads.h"
 #include "Capability.h"
 #include "Schedule.h"  /* to get at EMPTY_RUN_QUEUE() */
-#include "Signals.h" /* to get at handleSignalsInThisThread() */
 
 #if !defined(SMP)
 Capability MainCapability;     /* for non-SMP, we have one global capability */
@@ -155,7 +154,6 @@ grabCapability( Capability** cap )
   rts_n_free_capabilities = 0;
 #endif
   *cap = &MainCapability;
-  handleSignalsInThisThread();
 #else
   *cap = free_capabilities;
   free_capabilities = (*cap)->link;
@@ -277,7 +275,6 @@ waitForReturnCapability( Mutex* pMutex, Capability** pCap )
 #else
 	*pCap = &MainCapability;
 	ASSERT(rts_n_free_capabilities == 0);
-	handleSignalsInThisThread();
 #endif
     } else {
 	grabCapability(pCap);
