@@ -1,7 +1,6 @@
 /* -----------------------------------------------------------------------------
- * $Id: HsFFI.c,v 1.2 2003/01/28 16:30:06 simonmar Exp $
  *
- * (c) The GHC Team, 2002
+ * (c) The GHC Team, 2005
  *
  * RTS entry points as mandated by the FFI addendum to the Haskell 98 report
  *
@@ -15,8 +14,7 @@
 void
 hs_set_argv(int argc, char *argv[])
 {
-    prog_argc = argc;
-    prog_argv = argv;
+    setProgArgv(argc,argv);
 }
 
 void
@@ -27,7 +25,7 @@ hs_perform_gc(void)
 }
 
 void
-hs_free_stable_ptr(HsStablePtr *sp)
+hs_free_stable_ptr(HsStablePtr sp)
 {
     /* The cast is for clarity only, both HsStablePtr and StgStablePtr are
        typedefs for void*. */
@@ -35,7 +33,7 @@ hs_free_stable_ptr(HsStablePtr *sp)
 }
 
 void
-hs_free_fun_ptr(HsFunPtr *fp)
+hs_free_fun_ptr(HsFunPtr fp)
 {
     /* I simply *love* all these similar names... */
     freeHaskellFunctionPtr(fp);
