@@ -123,6 +123,15 @@ processHeapClosureForDead( StgClosure *c )
 	size = sizeofW(StgMVar);
 	return size;
 
+    case TVAR_WAIT_QUEUE:
+        return sizeofW(StgTVarWaitQueue);
+    case TVAR:
+        return sizeofW(StgTVar);
+    case TREC_CHUNK:
+        return sizeofW(StgTRecChunk);
+    case TREC_HEADER:
+        return sizeofW(StgTRecHeader);
+
     case MUT_ARR_PTRS:
     case MUT_ARR_PTRS_FROZEN:
 	size = mut_arr_ptrs_sizeW((StgMutArrPtrs *)c);
