@@ -322,18 +322,18 @@ printClosure( StgClosure *obj )
                 putchar(arrWordsGetChar(obj,i));
 		} */
 	    for (i=0; i<((StgArrWords *)obj)->words; i++)
-	      debugBelch("%u", ((StgArrWords *)obj)->payload[i]);
+	      debugBelch("%lu", ((StgArrWords *)obj)->payload[i]);
             debugBelch("\")\n");
             break;
         }
 
     case MUT_ARR_PTRS:
-	debugBelch("MUT_ARR_PTRS(size=%d)\n", ((StgMutArrPtrs *)obj)->ptrs);
+	debugBelch("MUT_ARR_PTRS(size=%ld)\n", ((StgMutArrPtrs *)obj)->ptrs);
 	break;
 
     case MUT_ARR_PTRS_FROZEN:
 #if !defined(XMLAMBDA)
-	debugBelch("MUT_ARR_PTRS_FROZEN(size=%d)\n", ((StgMutArrPtrs *)obj)->ptrs);
+	debugBelch("MUT_ARR_PTRS_FROZEN(size=%ld)\n", ((StgMutArrPtrs *)obj)->ptrs);
 	break;
 #else
           {
@@ -382,7 +382,7 @@ printClosure( StgClosure *obj )
             break;
 
     case STABLE_NAME:
-            debugBelch("STABLE_NAME(%d)\n", ((StgStableName*)obj)->sn); 
+            debugBelch("STABLE_NAME(%ld)\n", ((StgStableName*)obj)->sn); 
             break;
 
     case TSO:
@@ -499,12 +499,12 @@ printSmallBitmap( StgPtr spBottom, StgPtr payload, StgWord bitmap, nat size )
 
     p = payload;
     for(i = 0; i < size; i++, bitmap >>= 1 ) {
-	debugBelch("   stk[%d] (%p) = ", spBottom-(payload+i), payload+i);
+	debugBelch("   stk[%ld] (%p) = ", spBottom-(payload+i), payload+i);
 	if ((bitmap & 1) == 0) {
 	    printPtr((P_)payload[i]);
 	    debugBelch("\n");
 	} else {
-	    debugBelch("Word# %d\n", payload[i]);
+	    debugBelch("Word# %ld\n", payload[i]);
 	}
     }
 }
@@ -520,12 +520,12 @@ printLargeBitmap( StgPtr spBottom, StgPtr payload, StgLargeBitmap* large_bitmap,
 	StgWord bitmap = large_bitmap->bitmap[bmp];
 	j = 0;
 	for(; i < size && j < BITS_IN(W_); j++, i++, bitmap >>= 1 ) {
-	    debugBelch("   stk[%d] (%p) = ", spBottom-(payload+i), payload+i);
+	    debugBelch("   stk[%ld] (%p) = ", spBottom-(payload+i), payload+i);
 	    if ((bitmap & 1) == 0) {
 		printPtr((P_)payload[i]);
 		debugBelch("\n");
 	    } else {
-		debugBelch("Word# %d\n", payload[i]);
+		debugBelch("Word# %ld\n", payload[i]);
 	    }
 	}
     }
@@ -617,8 +617,8 @@ printStackChunk( StgPtr sp, StgPtr spBottom )
 	    switch (fun_info->f.fun_type) {
 	    case ARG_GEN:
 		printSmallBitmap(spBottom, sp+1,
-				 BITMAP_BITS(fun_info->f.bitmap),
-				 BITMAP_SIZE(fun_info->f.bitmap));
+				 BITMAP_BITS(fun_info->f.b.bitmap),
+				 BITMAP_SIZE(fun_info->f.b.bitmap));
 		break;
 	    case ARG_GEN_BIG:
 		printLargeBitmap(spBottom, sp+2,
