@@ -219,6 +219,12 @@ exitStorage (void)
     stat_exit(calcAllocated());
 }
 
+void
+freeStorage (void)
+{
+    freeAllMBlocks();
+}
+
 /* -----------------------------------------------------------------------------
    CAF management.
 
