@@ -1,16 +1,15 @@
 /* -----------------------------------------------------------------------------
- * $Id: Weak.h,v 1.4 1999/02/11 17:40:28 simonm Exp $
  *
- * (c) The GHC Team, 1998-1999
+ * (c) The GHC Team, 1998-2005
  *
  * Weak pointers / finalizers
  *
  * ---------------------------------------------------------------------------*/
 
-extern StgWeak *weak_ptr_list;
+#ifndef WEAK_H
+#define WEAK_H
 
-void finalizeWeakPointersNow(void);
 void scheduleFinalizers(StgWeak *w);
 void markWeakList(void);
 
-
+#endif
