@@ -1,5 +1,5 @@
 /* ------------------------------------------------------------------------
- * $Id: main.c,v 1.3 2004/08/13 13:11:23 simonmar Exp $
+ * $Id: main.c,v 1.3.6.1 2005/12/02 12:53:33 simonmar Exp $
  *									
  *	Copyright (C) 1995-2000 University of Oxford
  *									
@@ -33,6 +33,12 @@
 #include "matrix.h"
 #include "daVinci.h"
 
+#if HAVE_WINDOWS_H
+#include <windows.h>
+#define sleep(x) Sleep((x)*1000)
+#endif
+
+
 #define NoDeletes 80
 
 int     CriticalPath=CRITICAL_SYNCS;
