@@ -1,4 +1,6 @@
-module Layout ( LayoutItem(..), Root(..), Relative
+module Layout
+              {-# DEPRECATED "This module has been superseded by Graphics.UI.ObjectIO" #-}
+              ( LayoutItem(..), Root(..), Relative
               , getLayoutItem, layoutItems
               ) where
 
