@@ -1,5 +1,5 @@
 /* -----------------------------------------------------------------------------
- * $Id: HsPosix.h,v 1.6 2002/10/12 23:12:12 wolfgang Exp $
+ * $Id: HsPosix.h,v 1.7.4.1 2005/03/08 09:32:29 simonmar Exp $
  *
  * Definitions for package `posix' which are visible in Haskell land.
  *
@@ -67,11 +67,7 @@
 # endif
 #endif
 
-#ifdef HAVE_FRAMEWORK_HASKELLSUPPORT
-#include <HaskellSupport/dlfcn.h>
-#else
 #include <dlfcn.h>
-#endif
 
 #ifndef _POSIX_VDISABLE
 #define _POSIX_VDISABLE '\0'	/* Just a guess...but it works for Suns */
@@ -85,9 +81,7 @@ extern int  setenviron(char **);
 extern int  copyenv(void);
 extern int  _setenv(char *);
 extern int  delenv(char *);
-extern int  execvpe(char *, char **, char **);
 extern void stg_sigaddset(StgByteArray newset, StgByteArray oldset, int signum);
 extern void stg_sigdelset(StgByteArray newset, StgByteArray oldset, int signum);
-extern void pPrPr_disableITimers(void);
 
 #endif
