@@ -1,3 +1,7 @@
+#include <windows.h>
+#include "errors.h"
+#include "win32debug.h"
+#include "finalizers.h"
 #include "HsFFI.h"
 extern void* prim_createSolidBrush(DWORD arg1);
 extern HBRUSH access_prim_createSolidBrush_res1(HsPtr);
@@ -24,4 +28,4 @@ extern WORD prim_hOLLOW_BRUSH();
 extern void* prim_getStockBrush(WORD arg1);
 extern HBRUSH access_prim_getStockBrush_res1(HsPtr);
 extern int access_prim_getStockBrush_gc_failed(HsPtr);
-extern void* access_prim_getStockBrush_gc_failstring(HsPtr);
\ No newline at end of file
+extern void* access_prim_getStockBrush_gc_failstring(HsPtr);
