@@ -1,11 +1,11 @@
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Distribution.PackageDescription
--- Copyright   :  Isaac Jones 2003-2004
+-- Copyright   :  Isaac Jones 2003-2005
 -- 
 -- Maintainer  :  Isaac Jones <ijones@syntaxpolice.org>
 -- Stability   :  alpha
--- Portability :  
+-- Portability :  portable
 --
 -- Package description and parsing.
 
@@ -41,15 +41,15 @@ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. -}
 
 module Distribution.PackageDescription (
         -- * Package descriptions
-	PackageDescription(..),
-	emptyPackageDescription,
+        PackageDescription(..),
+        emptyPackageDescription,
         readPackageDescription,
-	parseDescription,
+        parseDescription,
         StanzaField(..),
         LineNo,
         basicStanzaFields,
         writePackageDescription,
-	showPackageDescription,
+        showPackageDescription,
         sanityCheckPackage, errorOut,
         setupMessage,
         Library(..),
@@ -63,48 +63,57 @@ module Distribution.PackageDescription (
         BuildInfo(..),
         emptyBuildInfo,
         -- ** Supplementary build information
-	HookedBuildInfo,
-	emptyHookedBuildInfo,
-	readHookedBuildInfo,
-	parseHookedBuildInfo,
-	writeHookedBuildInfo,
-	showHookedBuildInfo,        
-	updatePackageDescription,
+        HookedBuildInfo,
+        emptyHookedBuildInfo,
+        readHookedBuildInfo,
+        parseHookedBuildInfo,
+        writeHookedBuildInfo,
+        showHookedBuildInfo,        
+        updatePackageDescription,
         -- * Utilities
         ParseResult(..),
         PError, showError,
         hcOptions,
+        autogenModuleName,
 #ifdef DEBUG
         hunitTests,
         test
 #endif
   ) where
 
-import Control.Monad(liftM, foldM, when, unless)
+import Control.Monad(liftM, foldM, when)
 import Data.Char
 import Data.Maybe(fromMaybe, fromJust, isNothing, catMaybes)
+import Data.List (nub)
 import Text.PrettyPrint.HughesPJ
 import System.Directory(doesFileExist)
+import System.Environment(getProgName)
+import System.IO(hPutStrLn, stderr)
+import System.Exit
 
 import Distribution.ParseUtils
 import Distribution.Package(PackageIdentifier(..),showPackageId,
-			    parsePackageName)
-import Distribution.Version(Version(..), VersionRange(..),
-                            showVersion, parseVersion)
+                            parsePackageName)
+import Distribution.Version(Version(..), VersionRange(..), withinRange,
+                            showVersion, parseVersion, showVersionRange, parseVersionRange)
 import Distribution.License(License(..))
 import Distribution.Version(Dependency(..))
-import Distribution.Extension(Extension(..))
-import Distribution.Setup(CompilerFlavor(..))
-import Distribution.Simple.Utils(currentDir, die)
+import Distribution.Compiler(CompilerFlavor(..))
+import Distribution.Simple.Utils(currentDir, die, dieWithLocation, warn)
+import Language.Haskell.Extension(Extension(..))
 
 import Distribution.Compat.ReadP as ReadP hiding (get)
 
 #ifdef DEBUG
 import HUnit (Test(..), assertBool, Assertion, runTestTT, Counts, assertEqual)
-import Distribution.ParseUtils	(runP)
+import Distribution.ParseUtils  (runP)
 #endif
 
--- | This data type is the internal representation of the file @pkg.descr@.
+-- |Fix. Figure out a way to get this from .cabal file
+cabalVersion :: Version
+cabalVersion = Version [1,1,4] []
+
+-- | This data type is the internal representation of the file @pkg.cabal@.
 -- It contains two kinds of information about the package: information
 -- which is needed for all packages, such as the package name and version, and 
 -- information which is needed for the simple build system only, such as 
@@ -112,30 +121,36 @@ import Distribution.ParseUtils	(runP)
 -- 
 data PackageDescription
     =  PackageDescription {
-	-- the following are required by all packages:
-	package        :: PackageIdentifier,
+        -- the following are required by all packages:
+        package        :: PackageIdentifier,
         license        :: License,
         licenseFile    :: FilePath,
         copyright      :: String,
         maintainer     :: String,
-	author         :: String,
+        author         :: String,
         stability      :: String,
-	testedWith     :: [(CompilerFlavor,VersionRange)],
-	homepage       :: String,
-	pkgUrl         :: String,
-	synopsis       :: String,
-	description    :: String,
-	category       :: String,
+        testedWith     :: [(CompilerFlavor,VersionRange)],
+        homepage       :: String,
+        pkgUrl         :: String,
+        synopsis       :: String, -- ^A one-line summary of this package
+        description    :: String, -- ^A more verbose description of this package
+        category       :: String,
         buildDepends   :: [Dependency],
-	-- components
+        descCabalVersion :: VersionRange, -- ^If this package depends on a specific version of Cabal, give that here.
+        -- components
         library        :: Maybe Library,
-        executables    :: [Executable]
+        executables    :: [Executable],
+        dataFiles      :: [FilePath],
+        extraSrcFiles  :: [FilePath],
+        extraTmpFiles  :: [FilePath]
     }
     deriving (Show, Read, Eq)
 
-data Library = Library { exposedModules    :: [String],
-                         libBuildInfo      :: BuildInfo }
-             deriving (Show, Eq, Read)
+data Library = Library {
+        exposedModules    :: [String],
+        libBuildInfo      :: BuildInfo
+    }
+    deriving (Show, Eq, Read)
 
 emptyLibrary :: Library
 emptyLibrary = Library [] emptyBuildInfo
@@ -145,19 +160,23 @@ emptyPackageDescription
     =  PackageDescription {package      = PackageIdentifier "" (Version [] []),
                       license      = AllRightsReserved,
                       licenseFile  = "",
+                      descCabalVersion = AnyVersion,
                       copyright    = "",
                       maintainer   = "",
-		      author       = "",
+                      author       = "",
                       stability    = "",
-		      testedWith   = [],
+                      testedWith   = [],
                       buildDepends = [],
-		      homepage     = "",
-		      pkgUrl       = "",
-		      synopsis     = "",
-		      description  = "",
-		      category     = "",
+                      homepage     = "",
+                      pkgUrl       = "",
+                      synopsis     = "",
+                      description  = "",
+                      category     = "",
                       library      = Nothing,
-                      executables  = []
+                      executables  = [],
+                      dataFiles    = [],
+                      extraSrcFiles = [],
+                      extraTmpFiles = []
                      }
 
 -- |Get all the module names from the libraries in this package
@@ -180,34 +199,36 @@ data BuildInfo = BuildInfo {
         buildable         :: Bool,      -- ^ component is buildable here
         ccOptions         :: [String],  -- ^ options for C compiler
         ldOptions         :: [String],  -- ^ options for linker
-        frameworks        :: [String],
+        frameworks        :: [String], -- ^support frameworks for Mac OS X
         cSources          :: [FilePath],
-        hsSourceDir       :: FilePath,
-	otherModules      :: [String],
+        hsSourceDirs      :: [FilePath], -- ^ where to look for the haskell module hierarchy
+        otherModules      :: [String], -- ^ non-exposed or non-main modules
         extensions        :: [Extension],
-        extraLibs         :: [String],
+        extraLibs         :: [String], -- ^ what libraries to link with when compiling a program that uses your package
         extraLibDirs      :: [String],
-        includeDirs       :: [FilePath],
-        includes          :: [FilePath],
-        options           :: [(CompilerFlavor,[String])]
+        includeDirs       :: [FilePath], -- ^directories to find .h files
+        includes          :: [FilePath], -- ^ The .h files to be found in includeDirs
+        options           :: [(CompilerFlavor,[String])],
+        ghcProfOptions       :: [String]
     }
     deriving (Show,Read,Eq)
 
 emptyBuildInfo :: BuildInfo
 emptyBuildInfo = BuildInfo {
-		      buildable         = True,
-		      ccOptions         = [],
-		      ldOptions         = [],
-		      frameworks        = [],
-		      cSources          = [],
-		      hsSourceDir       = currentDir,
-		      otherModules      = [],
+                      buildable         = True,
+                      ccOptions         = [],
+                      ldOptions         = [],
+                      frameworks        = [],
+                      cSources          = [],
+                      hsSourceDirs      = [currentDir],
+                      otherModules      = [],
                       extensions        = [],
                       extraLibs         = [],
                       extraLibDirs      = [],
                       includeDirs       = [],
                       includes          = [],
-                      options           = []
+                      options           = [],
+                      ghcProfOptions       = []
                      }
 
 data Executable = Executable {
@@ -268,14 +289,18 @@ updatePackageDescription (mb_lib_bi, exe_bi) p
        --won't build anyway.  add to sanity checker?
       updateLibrary (Just bi) Nothing     = Just emptyLibrary{libBuildInfo=bi}
 
-      updateExecutables :: [(String, BuildInfo)] -> [Executable] -> [Executable]
+      updateExecutables :: [(String, BuildInfo)] -- ^[(exeName, new buildinfo)]
+                        -> [Executable]          -- ^list of executables to update
+                        -> [Executable]          -- ^list with exeNames updated
       updateExecutables exe_bi' executables' = foldr updateExecutable executables' exe_bi'
       
-      updateExecutable :: (String, BuildInfo) -> [Executable] -> [Executable]
+      updateExecutable :: (String, BuildInfo) -- ^(exeName, new buildinfo)
+                       -> [Executable]        -- ^list of executables to update
+                       -> [Executable]        -- ^libst with exeName updated
       updateExecutable _                 []         = []
       updateExecutable exe_bi'@(name,bi) (exe:exes)
         | exeName exe == name = exe{buildInfo = unionBuildInfo bi (buildInfo exe)} : exes
-        | otherwise           = updateExecutable exe_bi' exes
+        | otherwise           = exe : updateExecutable exe_bi' exes
 
 unionBuildInfo :: BuildInfo -> BuildInfo -> BuildInfo
 unionBuildInfo b1 b2
@@ -284,7 +309,7 @@ unionBuildInfo b1 b2
          ldOptions         = combine ldOptions,
          frameworks        = combine frameworks,
          cSources          = combine cSources,
-         hsSourceDir       = override hsSourceDir "hs-source-dir",
+         hsSourceDirs      = combine hsSourceDirs,
          otherModules      = combine otherModules,
          extensions        = combine extensions,
          extraLibs         = combine extraLibs,
@@ -295,20 +320,19 @@ unionBuildInfo b1 b2
         }
       where 
       combine :: (Eq a) => (BuildInfo -> [a]) -> [a]
-      combine f = f b1 ++ f b2
-      override :: (Eq a) => (BuildInfo -> a) -> String -> a
-      override f s
-	| v1 == def = v2
-	| v2 == def = v1
-	| otherwise = error $ "union: Two non-empty fields found in union attempt: " ++ s
-        where v1 = f b1
-	      v2 = f b2
-	      def = f emptyBuildInfo
+      combine f = nub $ f b1 ++ f b2
 
 -- |Select options for a particular Haskell compiler.
 hcOptions :: CompilerFlavor -> [(CompilerFlavor, [String])] -> [String]
 hcOptions hc hc_opts = [opt | (hc',opts) <- hc_opts, hc' == hc, opt <- opts]
 
+-- |The name of the auto-generated module associated with a package
+autogenModuleName :: PackageDescription -> String
+autogenModuleName pkg_descr =
+    "Paths_" ++ map fixchar (pkgName (package pkg_descr))
+  where fixchar '-' = '_'
+        fixchar c   = c
+
 -- ------------------------------------------------------------
 -- * Parsing & Pretty printing
 -- ------------------------------------------------------------
@@ -328,8 +352,11 @@ basicStanzaFields =
                            text                   parsePackageName
                            (pkgName . package)    (\name pkg -> pkg{package=(package pkg){pkgName=name}})
  , simpleField reqNameVersion
-                           (text . showVersion)   parseVersion 
+                           (text . showVersion)   parseVersion
                            (pkgVersion . package) (\ver pkg -> pkg{package=(package pkg){pkgVersion=ver}})
+ , simpleField "cabal-version"
+                           (text . showVersionRange) parseCabalVersion
+                           descCabalVersion       (\v pkg -> pkg{descCabalVersion=v})
  , simpleField "license"
                            (text . show)          parseLicenseQ
                            license                (\l pkg -> pkg{license=l})
@@ -369,12 +396,12 @@ basicStanzaFields =
  , listField "tested-with"
                            showTestedWith         parseTestedWithQ
                            testedWith             (\val pkg -> pkg{testedWith=val})
-
- , listField   "exposed-modules"
-                           text               parseModuleNameQ
-                           (\p -> maybe [] exposedModules (library p))
-                           (\xs    pkg -> let lib = fromMaybe emptyLibrary (library pkg) in
-                                              pkg{library = Just lib{exposedModules=xs}})
+ , listField "data-files"  showFilePath           parseFilePathQ
+                           dataFiles              (\val pkg -> pkg{dataFiles=val})
+ , listField "extra-source-files" showFilePath    parseFilePathQ
+                           extraSrcFiles          (\val pkg -> pkg{extraSrcFiles=val})
+ , listField "extra-tmp-files" showFilePath       parseFilePathQ
+                           extraTmpFiles          (\val pkg -> pkg{extraTmpFiles=val})
  ]
 
 executableStanzaFields :: [StanzaField Executable]
@@ -393,13 +420,13 @@ binfoFields =
                            (text . show)      parseReadS
                            buildable          (\val binfo -> binfo{buildable=val})
  , listField "cc-options"
-                           showToken	      parseTokenQ
+                           showToken          parseTokenQ
                            ccOptions          (\val binfo -> binfo{ccOptions=val})
  , listField "ld-options"
-                           showToken	      parseTokenQ
+                           showToken          parseTokenQ
                            ldOptions          (\val binfo -> binfo{ldOptions=val})
  , listField "frameworks"
-                           showToken	      parseTokenQ
+                           showToken          parseTokenQ
                            frameworks         (\val binfo -> binfo{frameworks=val})
  , listField   "c-sources"
                            showFilePath       parseFilePathQ
@@ -408,7 +435,7 @@ binfoFields =
                            (text . show)      parseExtensionQ
                            extensions         (\exts  binfo -> binfo{extensions=exts})
  , listField   "extra-libraries"
-                           showToken	      parseTokenQ
+                           showToken          parseTokenQ
                            extraLibs          (\xs    binfo -> binfo{extraLibs=xs})
  , listField   "extra-lib-dirs"
                            showFilePath       parseFilePathQ
@@ -419,12 +446,15 @@ binfoFields =
  , listField   "include-dirs"
                            showFilePath       parseFilePathQ
                            includeDirs        (\paths binfo -> binfo{includeDirs=paths})
- , simpleField "hs-source-dir"
+ , listField   "hs-source-dirs"
                            showFilePath       parseFilePathQ
-                           hsSourceDir        (\path  binfo -> binfo{hsSourceDir=path})
+                           hsSourceDirs       (\paths binfo -> binfo{hsSourceDirs=paths})
  , listField   "other-modules"         
                            text               parseModuleNameQ
                            otherModules       (\val binfo -> binfo{otherModules=val})
+ , listField   "ghc-prof-options"         
+                           text               parseTokenQ
+                           ghcProfOptions        (\val binfo -> binfo{ghcProfOptions=val})
  , optsField   "ghc-options"  GHC
                            options            (\path  binfo -> binfo{options=path})
  , optsField   "hugs-options" Hugs
@@ -445,8 +475,11 @@ readAndParseFile parser fpath = do
   when (not exists) (die $ "Error Parsing: file \"" ++ fpath ++ "\" doesn't exist. Cannot continue.")
   str <- readFile fpath
   case parser str of
-    ParseFailed e -> error (showError e) -- FIXME
+    ParseFailed e -> do
+        let (lineNo, message) = locatedErrorMsg e
+        dieWithLocation fpath lineNo message
     ParseOk x -> return x
+  where
 
 -- |Parse the given package file.
 readPackageDescription :: FilePath -> IO PackageDescription
@@ -464,24 +497,36 @@ parseDescription inp = do (st:sts) <- splitStanzas inp
         parseBasicStanza ((StanzaField name _ set):fields) pkg (lineNo, f, val)
           | name == f = set lineNo val pkg
           | otherwise = parseBasicStanza fields pkg (lineNo, f, val)
-        parseBasicStanza [] pkg (lineNo, f, val) = do
-          let lib = fromMaybe emptyLibrary (library pkg)
-	  bi <- parseBInfoField binfoFields (libBuildInfo lib) (lineNo, f, val)
-          return pkg{library=Just lib{libBuildInfo=bi}}
+          {-     
+     , listField   "exposed-modules"
+                           text               parseModuleNameQ
+                           (\p -> maybe [] exposedModules (library p))
+                           (\xs    pkg -> let lib = fromMaybe emptyLibrary (library pkg) in
+                                              pkg{library = Just lib{exposedModules=xs}})
+-}
+        parseBasicStanza [] pkg (lineNo, f, val)
+          | "exposed-modules" == f = do
+               mods <- runP lineNo f (parseOptCommaList parseModuleNameQ) val
+               return pkg{library=Just lib{exposedModules=mods}}
+          | otherwise = do
+               bi <- parseBInfoField binfoFields (libBuildInfo lib) (lineNo, f, val)
+               return pkg{library=Just lib{libBuildInfo=bi}}
+          where
+            lib = fromMaybe emptyLibrary (library pkg)
 
-        parseExecutableStanza st@((_, "executable",eName):_) =
+        parseExecutableStanza st@((lineNo, "executable",eName):_) =
           case lookupField "main-is" st of
-	    Just (_,_) -> foldM (parseExecutableField executableStanzaFields) emptyExecutable st
-	    Nothing           -> fail $ "No 'Main-Is' field found for " ++ eName ++ " stanza"
+            Just (_,_) -> foldM (parseExecutableField executableStanzaFields) emptyExecutable st
+            Nothing           -> syntaxError lineNo $ "No 'Main-Is' field found for " ++ eName ++ " stanza"
         parseExecutableStanza ((lineNo, f,_):_) = 
-          myError lineNo $ "'Executable' stanza starting with field '" ++ f ++ "'"
+          syntaxError lineNo $ "'Executable' stanza starting with field '" ++ f ++ "'"
         parseExecutableStanza _ = error "This shouldn't happen!"
 
         parseExecutableField ((StanzaField name _ set):fields) exe (lineNo, f, val)
-	  | name == f = set lineNo val exe
-	  | otherwise = parseExecutableField fields exe (lineNo, f, val)
-	parseExecutableField [] exe (lineNo, f, val) = do
-	  binfo <- parseBInfoField binfoFields (buildInfo exe) (lineNo, f, val)
+          | name == f = set lineNo val exe
+          | otherwise = parseExecutableField fields exe (lineNo, f, val)
+        parseExecutableField [] exe (lineNo, f, val) = do
+          binfo <- parseBInfoField binfoFields (buildInfo exe) (lineNo, f, val)
           return exe{buildInfo=binfo}
 
         -- ...
@@ -511,17 +556,17 @@ parseHookedBuildInfo inp = do
         | map toLower inFieldName == "executable"
             = do bis <- parseBI bi
                  return (mName, bis)
-        | otherwise = myError lineNo "expecting 'executable' at top of stanza"
-    parseExe [] = myError 0 "error in parsing buildinfo file. Expected executable stanza"
+        | otherwise = syntaxError lineNo "expecting 'executable' at top of stanza"
+    parseExe [] = syntaxError 0 "error in parsing buildinfo file. Expected executable stanza"
     parseBI :: Stanza -> ParseResult BuildInfo
     parseBI st = foldM (parseBInfoField binfoFields) emptyBuildInfo st
 
 parseBInfoField :: [StanzaField a] -> a -> (LineNo, String, String) -> ParseResult a
 parseBInfoField ((StanzaField name _ set):fields) binfo (lineNo, f, val)
-	  | name == f = set lineNo val binfo
-	  | otherwise = parseBInfoField fields binfo (lineNo, f, val)
+          | name == f = set lineNo val binfo
+          | otherwise = parseBInfoField fields binfo (lineNo, f, val)
 parseBInfoField [] _ (lineNo, f, _) =
-	  myError lineNo $ "Unknown field '" ++ f ++ "'"
+          syntaxError lineNo $ "Unknown field '" ++ f ++ "'"
 
 -- --------------------------------------------
 -- ** Pretty printing
@@ -534,7 +579,9 @@ showPackageDescription pkg = render $
   ppFields pkg basicStanzaFields $$
   (case library pkg of
      Nothing  -> empty
-     Just lib -> ppFields (libBuildInfo lib) binfoFields) $$
+     Just lib -> 
+        text "exposed-modules" <> colon <+> fsep (punctuate comma (map text (exposedModules lib))) $$
+        ppFields (libBuildInfo lib) binfoFields) $$
   vcat (map ppExecutable (executables pkg))
   where
     ppExecutable exe =
@@ -590,19 +637,24 @@ sanityCheckPackage pkg_descr
                       "Package is copyright All Rights Reserved"
           noLicenseFile = checkSanity (null $ licenseFile pkg_descr)
                           "No license-file field."
+          goodCabal = let v = (descCabalVersion pkg_descr)
+                          in checkSanity (not $ cabalVersion  `withinRange` v)
+                                 ("This package requires Cabal verion: " ++ (showVersionRange v) ++ ".")
 
          in return $ (catMaybes [nothingToDo, noModules,
                                  allRights, noLicenseFile]
-                     ,catMaybes $ libSane:(checkMissingFields pkg_descr))
+                     ,catMaybes $ libSane:goodCabal:(checkMissingFields pkg_descr))
 
 -- |Output warnings and errors. Exit if any errors.
 errorOut :: [String]  -- ^Warnings
          -> [String]  -- ^errors
          -> IO ()
 errorOut warnings errors = do
-  mapM (putStrLn . ("Warning: " ++)) warnings
-  mapM (putStrLn . ("Error: " ++))   errors
-  unless (null errors) (error "Errors detected. See above.")
+  mapM warn warnings
+  when (not (null errors)) $ do
+    pname <- getProgName
+    mapM (hPutStrLn stderr . ((pname ++ ": Error: ") ++)) errors
+    exitWith (ExitFailure 1)
 
 checkMissingFields :: PackageDescription -> [Maybe String]
 checkMissingFields pkg_descr = 
@@ -634,6 +686,11 @@ hasMods pkg_descr
     | otherwise = null (executables pkg_descr)
                    && null (exposedModules (fromJust (library pkg_descr)))
 
+parseCabalVersion = do v <- parseVersionRange
+                       if (cabalVersion `withinRange` v)
+                          then return v
+                          else error ("This package requires Cabal verion: " ++ (showVersionRange v) ++ ".")
+
 -- ------------------------------------------------------------
 -- * Testing
 -- ------------------------------------------------------------
@@ -646,6 +703,7 @@ testPkgDesc = unlines [
         "License: LGPL",
         "License-File: foo",
         "Copyright: Free Text String",
+        "Cabal-version: >1.1.1",
         "-- Optional - may be in source?",
         "Author: Happy Haskell Hacker",
         "Homepage: http://www.haskell.org/foo",
@@ -662,12 +720,14 @@ testPkgDesc = unlines [
         "Build-Depends: haskell-src, HUnit>=1.0.0-rain",
         "Other-Modules: Distribution.Package, Distribution.Version,",
         "                Distribution.Simple.GHCPackageConfig",
+        "Other-files: file1, file2",
+        "Extra-Tmp-Files:    file1, file2",
         "C-Sources: not/even/rain.c, such/small/hands",
-        "HS-Source-Dir: src",
+        "HS-Source-Dirs: src, src2",
         "Exposed-Modules: Distribution.Void, Foo.Bar",
         "Extensions: OverlappingInstances, TypeSynonymInstances",
         "Extra-Libraries: libfoo, bar, bang",
-	"Extra-Lib-Dirs: \"/usr/local/libs\"",
+        "Extra-Lib-Dirs: \"/usr/local/libs\"",
         "Include-Dirs: your/slightest, look/will",
         "Includes: /easily/unclose, /me, \"funky, path\\\\name\"",
         "GHC-Options: -fTH -fglasgow-exts",
@@ -699,6 +759,7 @@ testPkgDescAnswer =
                     synopsis = "a nice package!",
                     description = "a really nice package!",
                     category = "tools",
+                               descCabalVersion=LaterVersion (Version [1,1,1] []),
                     buildDepends = [Dependency "haskell-src" AnyVersion,
                                      Dependency "HUnit"
                                      (UnionVersionRanges (ThisVersion (Version [1,0,0] ["rain"]))
@@ -706,6 +767,9 @@ testPkgDescAnswer =
                     testedWith=[(GHC, AnyVersion)],
                     maintainer = "",
                     stability = "Free Text String",
+                    extraTmpFiles=["file1", "file2"],
+                    extraSrcFiles=["file1", "file2"],
+                    dataFiles=[],
 
                     library = Just $ Library {
                         exposedModules = ["Distribution.Void", "Foo.Bar"],
@@ -715,7 +779,7 @@ testPkgDescAnswer =
                            ldOptions = ["-BStatic", "-dn"],
                            frameworks = ["foo"],
                            cSources = ["not/even/rain.c", "such/small/hands"],
-                           hsSourceDir = "src",
+                           hsSourceDirs = ["src", "src2"],
                            otherModules = ["Distribution.Package",
                                            "Distribution.Version",
                                            "Distribution.Simple.GHCPackageConfig"],
@@ -725,13 +789,14 @@ testPkgDescAnswer =
                            includeDirs = ["your/slightest", "look/will"],
                            includes = ["/easily/unclose", "/me", "funky, path\\name"],
                            -- Note reversed order:
+                           ghcProfOptions = [],
                            options = [(NHC, []), (Hugs,["+TH"]), (GHC,["-fTH","-fglasgow-exts"])]}
                     },
                     executables = [Executable "somescript" 
                        "SomeFile.hs" (
                       emptyBuildInfo{
                         otherModules=["Foo1","Util","Main"],
-                        hsSourceDir = "scripts",
+                        hsSourceDirs = ["scripts"],
                         extensions = [OverlappingInstances],
                         options = [(NHC,[]),(Hugs,[]),(GHC,[])]
                       })]
@@ -790,7 +855,7 @@ comparePackageDescriptions :: PackageDescription
                            -> PackageDescription
                            -> [String]      -- ^Errors
 comparePackageDescriptions p1 p2
-    = catMaybes $ myCmp package "package" : myCmp license "license": myCmp licenseFile "licenseFile":  myCmp copyright "copyright":  myCmp maintainer "maintainer":  myCmp author "author":  myCmp stability "stability":  myCmp testedWith "testedWith":  myCmp homepage "homepage":  myCmp pkgUrl "pkgUrl":  myCmp synopsis "synopsis":  myCmp description "description":  myCmp category "category":  myCmp buildDepends "buildDepends":  myCmp library "library":  myCmp executables "executables": []
+    = catMaybes $ myCmp package "package" : myCmp license "license": myCmp licenseFile "licenseFile":  myCmp copyright "copyright":  myCmp maintainer "maintainer":  myCmp author "author":  myCmp stability "stability":  myCmp testedWith "testedWith":  myCmp homepage "homepage":  myCmp pkgUrl "pkgUrl":  myCmp synopsis "synopsis":  myCmp description "description":  myCmp category "category":  myCmp buildDepends "buildDepends":  myCmp library "library":  myCmp executables "executables": myCmp descCabalVersion "cabal-version":[]
 
 
       where myCmp :: (Eq a, Show a) => (PackageDescription -> a)
