@@ -5,7 +5,7 @@
 -- 
 -- Maintainer  :  Isaac Jones <ijones@syntaxpolice.org>
 -- Stability   :  alpha
--- Portability :  GHC, Hugs
+-- Portability :  portable
 --
 -- Explanation: Data types and parser for the standard command-line
 -- setup.  Will also return commands it doesn't know about.
@@ -41,56 +41,54 @@ THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. -}
 
 module Distribution.Setup (--parseArgs,
-                           Action(..), ConfigFlags(..),
-                           CopyFlags, InstallFlags, RegisterFlags,
-                           CompilerFlavor(..), Compiler(..),
+                           module Distribution.Compiler,
+                           Action(..),
+                           ConfigFlags(..), emptyConfigFlags, configureArgs,
+                           CopyFlags, CopyDest(..), InstallFlags, RegisterFlags, SDistFlags,
+                           InstallUserFlag(..),
 			   --optionHelpString,
 #ifdef DEBUG
                            hunitTests,
 #endif
-                           parseGlobalArgs,
+                           parseGlobalArgs, defaultCompilerFlavor,
                            parseConfigureArgs, parseBuildArgs, parseCleanArgs,
-                           parseHaddockArgs, parseProgramaticaArgs,
+                           parseHaddockArgs, parseProgramaticaArgs, parseTestArgs,
                            parseInstallArgs, parseSDistArgs, parseRegisterArgs,
-                           parseUnregisterArgs, parseCopyArgs
+                           parseUnregisterArgs, parseCopyArgs,
+                           reqPathArg, reqDirArg
                            ) where
 
+
 -- Misc:
 #ifdef DEBUG
 import HUnit (Test(..))
 #endif
 
-import Control.Monad(when)
-import Distribution.Version (Version)
+import Distribution.Compiler (CompilerFlavor(..), Compiler(..))
+import Distribution.Simple.Utils (die)
+import Distribution.Program(ProgramLocation(..), ProgramConfiguration(..),
+                            Program(..),
+                            defaultProgramConfiguration, userSpecifyPath,
+                            userSpecifyArgs, haddockProgram)
 import Data.List(find)
+import Data.FiniteMap(keysFM)
 import Distribution.GetOpt
+import Distribution.Compat.FilePath (platformPath)
 import System.Exit
 import System.Environment
 
--- ------------------------------------------------------------
--- * Command Line Types and Exports
--- ------------------------------------------------------------
-
-data CompilerFlavor = GHC | NHC | Hugs | HBC | Helium | OtherCompiler String
-              deriving (Show, Read, Eq)
-
-data Compiler = Compiler {compilerFlavor:: CompilerFlavor,
-			  compilerVersion :: Version,
-                          compilerPath  :: FilePath,
-                          compilerPkgTool :: FilePath}
-                deriving (Show, Read, Eq)
-
 -- type CommandLineOpts = (Action,
 --                         [String]) -- The un-parsed remainder
 
 data Action = ConfigCmd ConfigFlags       -- config
             | BuildCmd                    -- build
             | CleanCmd                    -- clean
-            | CopyCmd (Maybe FilePath)    -- copy
+            | CopyCmd CopyDest            -- copy (--destdir flag)
             | HaddockCmd                  -- haddock
             | ProgramaticaCmd             -- pfesetup
-            | InstallCmd Bool -- install (install-prefix) (--user flag)
+            | InstallCmd InstallUserFlag -- install (install-prefix) (--user flag)
             | SDistCmd                    -- sdist
+            | TestCmd                     -- test
             | RegisterCmd   Bool Bool     -- register (--user flag, --gen-script)
             | UnregisterCmd Bool Bool     -- unregister (--user flag, --gen-script)
 	    | HelpCmd			  -- help
@@ -99,50 +97,135 @@ data Action = ConfigCmd ConfigFlags       -- config
 --             | BDist -- 1.0
 --            | CleanCmd                 -- clean
 --            | NoCmd -- error case?
-    deriving (Show, Eq)
 
 -- | Flags to @configure@ command
 data ConfigFlags = ConfigFlags {
+        configPrograms :: ProgramConfiguration, -- ^All programs that cabal may run
         configHcFlavor :: Maybe CompilerFlavor,
         configHcPath   :: Maybe FilePath, -- ^given compiler location
         configHcPkg    :: Maybe FilePath, -- ^given hc-pkg location
-        configHaddock  :: Maybe FilePath, -- ^Haddock path
         configHappy    :: Maybe FilePath, -- ^Happy path
         configAlex     :: Maybe FilePath, -- ^Alex path
         configHsc2hs   :: Maybe FilePath, -- ^Hsc2hs path
+        configC2hs     :: Maybe FilePath, -- ^C2hs path
         configCpphs    :: Maybe FilePath, -- ^Cpphs path
-        configPrefix   :: Maybe FilePath, -- ^installation prefix
-        configVerbose  :: Int,            -- ^verbosity level
-	configUser     :: Bool		  -- ^--user flag?
-    }
-    deriving (Show, Eq)
+        configGreencard:: Maybe FilePath, -- ^GreenCard path
+        configProfLib  :: Bool,           -- ^Enable profiling in the library
+        configProfExe  :: Bool,           -- ^Enable profiling in the executables.
+        configPrefix   :: Maybe FilePath,
+		-- ^installation prefix
+	configBinDir   :: Maybe FilePath, 
+		-- ^installation dir for binaries,
+	configLibDir   :: Maybe FilePath, 
+		-- ^installation dir for object code libraries, 
+	configLibSubDir :: Maybe FilePath,
+		-- ^subdirectory of libdir in which libs are installed
+	configLibExecDir :: Maybe FilePath,
+		-- ^installation dir for program executables,
+	configDataDir  :: Maybe FilePath,
+		-- ^installation dir for read-only arch-independent data,
+	configDataSubDir :: Maybe FilePath,
+		-- ^subdirectory of datadir in which data files are installed
 
-emptyConfigFlags :: ConfigFlags
-emptyConfigFlags = ConfigFlags {
-        configHcFlavor = Nothing,
+        configVerbose  :: Int,            -- ^verbosity level
+	configUser     :: Bool,		  -- ^--user flag?
+	configGHCiLib  :: Bool            -- ^Enable compiling library for GHCi
+    }
+
+emptyConfigFlags :: ProgramConfiguration -> ConfigFlags
+emptyConfigFlags progConf = ConfigFlags {
+        configPrograms = progConf,
+        configHcFlavor = defaultCompilerFlavor,
         configHcPath   = Nothing,
         configHcPkg    = Nothing,
-        configHaddock  = Nothing,
+--        configHaddock  = EmptyLocation,
         configHappy    = Nothing,
         configAlex     = Nothing,
         configHsc2hs   = Nothing,
+        configC2hs     = Nothing,
+        configProfLib  = False,
+        configProfExe  = False,
         configCpphs    = Nothing,
+        configGreencard= Nothing,
         configPrefix   = Nothing,
+	configBinDir   = Nothing,
+	configLibDir   = Nothing,
+	configLibSubDir = Nothing,
+	configLibExecDir = Nothing,
+	configDataDir  = Nothing,
+	configDataSubDir = Nothing,
         configVerbose  = 0,
-	configUser     = False
+	configUser     = False,
+	configGHCiLib  = True
     }
 
+defaultCompilerFlavor :: Maybe CompilerFlavor
+defaultCompilerFlavor =
+#if defined(__GLASGOW_HASKELL__)
+   Just GHC
+#elif defined(__NHC__)
+   Just NHC
+#elif defined(__HUGS__)
+   Just Hugs
+#else
+   Nothing
+#endif
+
+-- | Arguments to pass to a @configure@ script, e.g. generated by
+-- @autoconf@.
+configureArgs :: ConfigFlags -> [String]
+configureArgs flags
+  = hc_flag ++
+        optFlag "with-hc-pkg" configHcPkg ++
+        optFlag "prefix" configPrefix ++
+        optFlag "bindir" configBinDir ++
+        optFlag "libdir" configLibDir ++
+        optFlag "libexecdir" configLibExecDir ++
+        optFlag "datadir" configDataDir
+  where
+        hc_flag = case (configHcFlavor flags, configHcPath flags) of
+                        (_, Just hc_path)  -> ["--with-hc=" ++ hc_path]
+                        (Just hc, Nothing) -> ["--with-hc=" ++ showHC hc]
+                        (Nothing,Nothing)  -> []
+        optFlag name config_field = case config_field flags of
+                        Just p -> ["--" ++ name ++ "=" ++ p]
+                        Nothing -> []
+
+        showHC GHC = "ghc"
+        showHC NHC = "nhc98"
+        showHC Hugs = "hugs"
+        showHC c    = "unknown compiler: " ++ (show c)
+
 -- |Most of these flags are for Configure, but InstPrefix is for Copy.
 data Flag a = GhcFlag | NhcFlag | HugsFlag
-          | WithCompiler FilePath | WithHcPkg FilePath | Prefix FilePath
-          | WithHaddock FilePath | WithHappy FilePath | WithAlex FilePath
-          | WithHsc2hs FilePath | WithCpphs FilePath
+          | WithCompiler FilePath | WithHcPkg FilePath
+          | WithHappy FilePath | WithAlex FilePath
+          | WithHsc2hs FilePath | WithC2hs FilePath | WithCpphs FilePath
+          | WithGreencard FilePath
+          | WithProfLib | WithoutProfLib
+          | WithProfExe | WithoutProfExe
+	  | WithGHCiLib | WithoutGHCiLib
+
+	  | Prefix FilePath
+	  | BinDir FilePath
+	  | LibDir FilePath
+	  | LibSubDir FilePath
+	  | LibExecDir FilePath
+	  | DataDir FilePath
+	  | DataSubDir FilePath
+
+          | ProgramArgs String String   -- program name, arguments
+          | WithProgram String FilePath -- program name, location
+
           -- For install, register, and unregister:
           | UserFlag | GlobalFlag
           -- for register & unregister
           | GenScriptFlag
           -- For copy:
           | InstPrefix FilePath
+	  | DestDir FilePath
+          -- For sdist:
+          | Snapshot
           -- For everyone:
           | HelpFlag
           | Verbose Int
@@ -171,9 +254,6 @@ liftCustomOpts flags = [ Option shopt lopt (f adesc) help
         f (ReqArg g s) = ReqArg (Lift . g) s
         f (OptArg g s) = OptArg (Lift . g) s
 
-unliftFlags :: [Flag a] -> [a]
-unliftFlags flags = [ fl | Lift fl <- flags ]
-
 data Cmd a = Cmd {
         cmdName         :: String,
         cmdHelp         :: String, -- Short description
@@ -182,25 +262,26 @@ data Cmd a = Cmd {
         cmdAction       :: Action
         }
 
-commandList :: [Cmd a]
-commandList = [configureCmd, buildCmd, cleanCmd, installCmd,
-               copyCmd, sdistCmd, haddockCmd, programaticaCmd,
-               registerCmd, unregisterCmd]
+commandList :: ProgramConfiguration -> [Cmd a]
+commandList progConf = [(configureCmd progConf), buildCmd, cleanCmd, installCmd,
+                        copyCmd, sdistCmd, testCmd, haddockCmd, programaticaCmd,
+                        registerCmd, unregisterCmd]
 
 lookupCommand :: String -> [Cmd a] -> Maybe (Cmd a)
 lookupCommand name = find ((==name) . cmdName)
 
-printGlobalHelp :: IO ()
-printGlobalHelp = do pname <- getProgName
+printGlobalHelp :: ProgramConfiguration -> IO ()
+printGlobalHelp progConf = 
+                  do pname <- getProgName
                      let syntax_line = "Usage: " ++ pname ++ " [GLOBAL FLAGS]\n  or:  " ++ pname ++ " COMMAND [FLAGS]\n\nGlobal flags:"
                      putStrLn (usageInfo syntax_line globalOptions)
                      putStrLn "Commands:"
-                     let maxlen = maximum [ length (cmdName cmd) | cmd <- commandList ]
+                     let maxlen = maximum [ length (cmdName cmd) | cmd <- (commandList progConf) ]
                      sequence_ [ do putStr "  "
                                     putStr (align maxlen (cmdName cmd))
                                     putStr "    "
                                     putStrLn (cmdHelp cmd)
-                               | cmd <- commandList ]
+                               | cmd <- (commandList progConf) ]
                      putStrLn $ "\nFor more information about a command, try '" ++ pname ++ " COMMAND --help'."
   where align n str = str ++ replicate (n - length str) ' '
 
@@ -218,25 +299,21 @@ getCmdOpt cmd opts s = let (a,_,c,d) = getOpt' Permute (cmdOptions cmd ++ liftCu
 hasHelpFlag :: [Flag a] -> Bool
 hasHelpFlag flags = not . null $ [ () | HelpFlag <- flags ]
 
-parseGlobalArgs :: [String] -> IO (Action,[String])
-parseGlobalArgs args =
+parseGlobalArgs :: ProgramConfiguration -> [String] -> IO (Action,[String])
+parseGlobalArgs progConf args =
   case getOpt' RequireOrder globalOptions args of
     (flags, _, _, []) | hasHelpFlag flags -> do
-      printGlobalHelp
+      (printGlobalHelp progConf)
       exitWith ExitSuccess
     (_, cname:cargs, _, []) -> do
-      case lookupCommand cname commandList of
+      case lookupCommand cname (commandList progConf) of
         Just cmd -> return (cmdAction cmd,cargs)
-        Nothing  -> do putStrLn $ "Unrecognised command: " ++ cname ++ " (try --help)"
-                       exitWith (ExitFailure 1)
-    (_, [], _, [])  -> do putStrLn $ "No command given (try --help)"
-                          exitWith (ExitFailure 1)
-    (_, _, _, errs) -> do putStrLn "Errors:"
-                          mapM_ putStrLn errs
-                          exitWith (ExitFailure 1)
+        Nothing  -> die $ "Unrecognised command: " ++ cname ++ " (try --help)"
+    (_, [], _, [])  -> die $ "No command given (try --help)"
+    (_, _, _, errs) -> putErrors errs
 
-configureCmd :: Cmd a
-configureCmd = Cmd {
+configureCmd :: ProgramConfiguration -> Cmd a
+configureCmd progConf = Cmd {
         cmdName        = "configure",
         cmdHelp        = "Prepare to build the package.",
         cmdDescription = "",  -- This can be a multi-line description
@@ -244,61 +321,119 @@ configureCmd = Cmd {
            Option "g" ["ghc"] (NoArg GhcFlag) "compile with GHC",
            Option "n" ["nhc"] (NoArg NhcFlag) "compile with NHC",
            Option "" ["hugs"] (NoArg HugsFlag) "compile with hugs",
-           Option "w" ["with-compiler"] (ReqArg WithCompiler "PATH")
+           Option "w" ["with-compiler"] (reqPathArg WithCompiler)
                "give the path to a particular compiler",
-           Option "" ["with-hc-pkg"] (ReqArg WithHcPkg "PATH")
+           Option "" ["with-hc-pkg"] (reqPathArg WithHcPkg)
                "give the path to the package tool",
-           Option "" ["prefix"] (ReqArg Prefix "DIR")
+           Option "" ["prefix"] (reqDirArg Prefix)
                "bake this prefix in preparation of installation",
-           Option "" ["with-haddock"] (ReqArg WithHaddock "PATH")
-               "give the path to haddock",
-           Option "" ["with-happy"] (ReqArg WithHappy "PATH")
+	   Option "" ["bindir"] (reqDirArg BinDir)
+		"installation directory for executables",
+	   Option "" ["libdir"] (reqDirArg LibDir)
+		"installation directory for libraries",
+	   Option "" ["libsubdir"] (reqDirArg LibSubDir)
+		"subdirectory of libdir in which libs are installed",
+	   Option "" ["libexecdir"] (reqDirArg LibExecDir)
+		"installation directory for program executables",
+	   Option "" ["datadir"] (reqDirArg DataDir)
+		"installation directory for read-only data",
+	   Option "" ["datasubdir"] (reqDirArg DataSubDir)
+		"subdirectory of datadir in which data files are installed",
+           Option "" ["with-happy"] (reqPathArg WithHappy)
                "give the path to happy",
-           Option "" ["with-alex"] (ReqArg WithAlex "PATH")
+           Option "" ["with-alex"] (reqPathArg WithAlex)
                "give the path to alex",
-           Option "" ["with-hsc2hs"] (ReqArg WithHsc2hs "PATH")
+           Option "" ["with-hsc2hs"] (reqPathArg WithHsc2hs)
                "give the path to hsc2hs",
-           Option "" ["with-cpphs"] (ReqArg WithCpphs "PATH")
+           Option "" ["with-c2hs"] (reqPathArg WithC2hs)
+               "give the path to c2hs",
+           Option "" ["with-cpphs"] (reqPathArg WithCpphs)
                "give the path to cpphs",
+           Option "" ["with-greencard"] (reqPathArg WithGreencard)
+               "give the path to greencard",
+           Option "p" ["enable-library-profiling"] (NoArg WithProfLib)
+               "Enable library profiling",
+           Option "" ["disable-library-profiling"] (NoArg WithoutProfLib)
+               "Disable library profiling",
+           Option "" ["enable-executable-profiling"] (NoArg WithProfExe)
+               "Enable executable profiling",
+           Option "" ["disable-executable-profiling"] (NoArg WithoutProfExe)
+               "Disable executable profiling",
+	   Option "" ["enable-library-for-ghci"] (NoArg WithGHCiLib)
+               "compile library for use with GHCi",
+	   Option "" ["disable-library-for-ghci"] (NoArg WithoutGHCiLib)
+               "do not compile libraries for GHCi",
            Option "" ["user"] (NoArg UserFlag)
-               "allow dependencies to be satisfied from the user package database",
+               "allow dependencies to be satisfied from the user package database. also implies install --user",
            Option "" ["global"] (NoArg GlobalFlag)
                "(default) dependencies must be satisfied from the global package database"
-           ],
-        cmdAction      = ConfigCmd emptyConfigFlags
+           ]
+{- 
+   FIX: Instead of using ++ here, we might add extra arguments.  That
+   way, we can condense the help out put to something like
+   --with-{haddock,happy,alex,etc}
+
+   FIX: shouldn't use default. Look in hooks?.
+-}
+         ++ (withProgramOptions progConf)
+         ++ (programArgsOptions progConf),
+        cmdAction      = ConfigCmd (emptyConfigFlags progConf)
         }
 
-parseConfigureArgs :: ConfigFlags -> [String] -> [OptDescr a] ->
+programArgsOptions :: ProgramConfiguration -> [OptDescr (Flag a)]
+programArgsOptions (ProgramConfiguration conf) = map f (keysFM conf)
+    where f name = Option "" [name ++ "-args"] (reqPathArg (ProgramArgs name))
+                   ("give the args to " ++ name)
+
+withProgramOptions :: ProgramConfiguration -> [OptDescr (Flag a)]
+withProgramOptions (ProgramConfiguration conf) = map f (keysFM conf)
+    where f name = Option "" ["with-" ++ name] (reqPathArg (WithProgram name))
+                   ("give the path to " ++ name)
+
+reqPathArg :: (FilePath -> a) -> ArgDescr a
+reqPathArg constr = ReqArg (constr . platformPath) "PATH"
+
+reqDirArg :: (FilePath -> a) -> ArgDescr a
+reqDirArg constr = ReqArg (constr . platformPath) "DIR"
+
+parseConfigureArgs :: ProgramConfiguration -> ConfigFlags -> [String] -> [OptDescr a] ->
                       IO (ConfigFlags, [a], [String])
-parseConfigureArgs cfg args customOpts =
-  case getCmdOpt configureCmd customOpts args of
-    (flags, _, []) | hasHelpFlag flags -> do
-      printCmdHelp configureCmd customOpts
-      exitWith ExitSuccess
-    (flags, args', []) ->
-      return (updateCfg flags cfg, unliftFlags flags, args')
-    (_, _, errs) -> do putStrLn "Errors: "
-                       mapM_ putStrLn errs
-                       exitWith (ExitFailure 1)
-  where updateCfg (fl:flags) t = updateCfg flags $
-          case fl of
-            GhcFlag           -> t { configHcFlavor = Just GHC }
-            NhcFlag           -> t { configHcFlavor = Just NHC }
-            HugsFlag          -> t { configHcFlavor = Just Hugs }
-            WithCompiler path -> t { configHcPath   = Just path }
-            WithHcPkg path    -> t { configHcPkg    = Just path }
-            WithHaddock path  -> t { configHaddock  = Just path }
-            WithHappy path    -> t { configHappy    = Just path }
-            WithAlex path     -> t { configAlex     = Just path }
-            WithHsc2hs path   -> t { configHsc2hs   = Just path }
-            WithCpphs path    -> t { configCpphs    = Just path }
-            Prefix path       -> t { configPrefix   = Just path }
-            Verbose n         -> t { configVerbose  = n }
-	    UserFlag	      -> t { configUser     = True }
-	    GlobalFlag	      -> t { configUser     = False }
-            Lift _            -> t
-            _                 -> error $ "Unexpected flag!"
-        updateCfg [] t = t
+parseConfigureArgs progConf = parseArgs (configureCmd progConf) updateCfg
+  where updateCfg t GhcFlag              = t { configHcFlavor = Just GHC }
+        updateCfg t NhcFlag              = t { configHcFlavor = Just NHC }
+        updateCfg t HugsFlag             = t { configHcFlavor = Just Hugs }
+        updateCfg t (WithCompiler path)  = t { configHcPath   = Just path }
+        updateCfg t (WithHcPkg path)     = t { configHcPkg    = Just path }
+        updateCfg t (WithHappy path)     = t { configHappy    = Just path }
+        updateCfg t (WithAlex path)      = t { configAlex     = Just path }
+        updateCfg t (WithHsc2hs path)    = t { configHsc2hs   = Just path }
+        updateCfg t (WithC2hs path)      = t { configC2hs     = Just path }
+        updateCfg t (WithCpphs path)     = t { configCpphs    = Just path }
+        updateCfg t (WithGreencard path) = t { configGreencard= Just path }
+        updateCfg t (ProgramArgs name args) = t { configPrograms = (userSpecifyArgs
+                                                                 name
+                                                                 args (configPrograms t))}
+        updateCfg t (WithProgram name path) = t { configPrograms = (userSpecifyPath
+                                                                 name
+                                                                 path (configPrograms t))}
+        updateCfg t WithProfLib          = t { configProfLib  = True }
+        updateCfg t WithoutProfLib       = t { configProfLib  = False }
+        updateCfg t WithProfExe          = t { configProfExe  = True }
+        updateCfg t WithoutProfExe       = t { configProfExe  = False }
+	updateCfg t WithGHCiLib          = t { configGHCiLib  = True }
+	updateCfg t WithoutGHCiLib       = t { configGHCiLib  = False }
+        updateCfg t (Prefix path)        = t { configPrefix   = Just path }
+        updateCfg t (BinDir path)        = t { configBinDir   = Just path }
+        updateCfg t (LibDir path)        = t { configLibDir   = Just path }
+        updateCfg t (LibSubDir path)     = t { configLibSubDir= Just path }
+        updateCfg t (LibExecDir path)    = t { configLibExecDir = Just path }
+        updateCfg t (DataDir path)       = t { configDataDir  = Just path }
+        updateCfg t (DataSubDir path)    = t { configDataSubDir = Just path }
+        updateCfg t (Verbose n)          = t { configVerbose  = n }
+        updateCfg t UserFlag             = t { configUser     = True }
+        updateCfg t GlobalFlag           = t { configUser     = False }
+        updateCfg t (Lift _)             = t
+        updateCfg t _                    = error $ "Unexpected flag!"
 
 buildCmd :: Cmd a
 buildCmd = Cmd {
@@ -354,91 +489,98 @@ installCmd = Cmd {
         cmdHelp        = "Copy the files into the install locations. Run register.",
         cmdDescription = "Unlike the copy command, install calls the register command.\nIf you want to install into a location that is not what was\nspecified in the configure step, use the copy command.\n",
         cmdOptions     = [cmd_help, cmd_verbose,
-           Option "" ["install-prefix"] (ReqArg InstPrefix "DIR")
+           Option "" ["install-prefix"] (reqDirArg InstPrefix)
                "[DEPRECATED, use copy]",
            Option "" ["user"] (NoArg UserFlag)
                "upon registration, register this package in the user's local package database",
            Option "" ["global"] (NoArg GlobalFlag)
-               "(default) upon registration, register this package in the system-wide package database"
+               "(default; override with configure) upon registration, register this package in the system-wide package database"
            ],
-        cmdAction      = InstallCmd False
+        cmdAction      = InstallCmd InstallUserNone
         }
 
 copyCmd :: Cmd a
 copyCmd = Cmd {
         cmdName        = "copy",
         cmdHelp        = "Copy the files into the install locations.",
-        cmdDescription = "Does not call register, and allows a prefix at install time\nWithout the copy-prefix flag, configure determines location.\n",
+        cmdDescription = "Does not call register, and allows a prefix at install time\nWithout the --destdir flag, configure determines location.\n",
         cmdOptions     = [cmd_help, cmd_verbose,
-           Option "" ["copy-prefix"] (ReqArg InstPrefix "DIR")
-               "specify the directory in which to place installed files"
+           Option "" ["destdir"] (reqDirArg DestDir)
+               "directory to copy files to, prepended to installation directories",
+           Option "" ["copy-prefix"] (reqDirArg InstPrefix)
+               "[DEPRECATED, directory to copy files to instead of prefix]"
            ],
-        cmdAction      = CopyCmd Nothing
+        cmdAction      = CopyCmd NoCopyDest
         }
 
--- | Flags to @copy@: (Copy Location, verbose)
-type CopyFlags = (Maybe FilePath,Int)
+-- | Flags to @copy@: (destdir, copy-prefix (backwards compat), verbose)
+type CopyFlags = (CopyDest,Int)
+
+data CopyDest
+  = NoCopyDest
+  | CopyTo FilePath
+  | CopyPrefix FilePath		-- DEPRECATED
+  deriving (Eq, Show)
+
+data InstallUserFlag = InstallUserNone   -- ^no --user OR --global flag.
+                      | InstallUserUser   -- ^--user flag
+                      | InstallUserGlobal -- ^--global flag
 
 parseCopyArgs :: CopyFlags -> [String] -> [OptDescr a] ->
                     IO (CopyFlags, [a], [String])
-parseCopyArgs cfg args customOpts =
-  case getCmdOpt copyCmd customOpts args of
-    (flags, _, []) | hasHelpFlag flags -> do
-      printCmdHelp copyCmd customOpts
-      exitWith ExitSuccess
-    (flags, args', []) ->
-      return (updateCfg flags cfg, unliftFlags flags, args')
-    (_, _, errs) -> do putStrLn "Errors: "
-                       mapM_ putStrLn errs
-                       exitWith (ExitFailure 1)
-  where updateCfg (fl:flags) (mprefix,verbose) = updateCfg flags $
-          case fl of
-            InstPrefix path -> (Just path,verbose)
-            Verbose n       -> (mprefix,n)
-            Lift _          -> (mprefix,verbose)
+parseCopyArgs = parseArgs copyCmd updateCfg
+  where updateCfg (copydest,verbose) fl = case fl of
+            InstPrefix path -> (CopyPrefix path, verbose)
+	    DestDir path    -> (CopyTo path,verbose)
+            Verbose n       -> (copydest,n)
             _               -> error $ "Unexpected flag!"
-        updateCfg [] t = t
 
 -- | Flags to @install@: (user package, verbose)
-type InstallFlags = (Bool,Int)
+type InstallFlags = (InstallUserFlag,Int)
 
 parseInstallArgs :: InstallFlags -> [String] -> [OptDescr a] ->
                     IO (InstallFlags, [a], [String])
-parseInstallArgs cfg args customOpts =
-  case getCmdOpt installCmd customOpts args of
-    (flags, _, []) | hasHelpFlag flags -> do
-      printCmdHelp installCmd customOpts
-      exitWith ExitSuccess
-    (flags, args', []) ->
-      when (any isInstallPref flags) (error "--install-prefix is deprecated. Use copy command instead.") >>
-      return (updateCfg flags cfg, unliftFlags flags, args')
-    (_, _, errs) -> do putStrLn "Errors: "
-                       mapM_ putStrLn errs
-                       exitWith (ExitFailure 1)
-  where updateCfg :: [Flag a] -> (Bool,Int) -> (Bool,Int)
-        updateCfg (fl:flags) (uFlag,verbose) = updateCfg flags $
-          case fl of
-            InstPrefix _ -> error "--install-prefix is deprecated. Use copy command instead."
-            UserFlag     -> (True,verbose)
-            GlobalFlag   -> (False,verbose)
-            Verbose n    -> (uFlag,n)
-            Lift _       -> (uFlag,verbose)
+parseInstallArgs = parseArgs installCmd updateCfg
+  where updateCfg (uFlag,verbose) fl = case fl of
+            InstPrefix _ -> error "--install-prefix is obsolete. Use copy command instead."
+            UserFlag     -> (InstallUserUser,  verbose)
+            GlobalFlag   -> (InstallUserGlobal, verbose)
+            Verbose n    -> (uFlag, n)
             _            -> error $ "Unexpected flag!"
-        updateCfg [] t = t
-        isInstallPref (InstPrefix _) = True
-        isInstallPref _              = False
 
 sdistCmd :: Cmd a
 sdistCmd = Cmd {
         cmdName        = "sdist",
         cmdHelp        = "Generate a source distribution file (.tar.gz or .zip).",
         cmdDescription = "",  -- This can be a multi-line description
-        cmdOptions     = [cmd_help,cmd_verbose],
+        cmdOptions     = [cmd_help,cmd_verbose,
+           Option "" ["snapshot"] (NoArg Snapshot)
+               "Produce a snapshot source distribution"
+           ],
         cmdAction      = SDistCmd
         }
 
-parseSDistArgs :: [String] -> [OptDescr a] -> IO (Int, [a], [String])
-parseSDistArgs = parseNoArgs sdistCmd
+-- | Flags to @sdist@: (snapshot, verbose)
+type SDistFlags = (Bool, Int)
+
+parseSDistArgs :: [String] -> [OptDescr a] -> IO (SDistFlags, [a], [String])
+parseSDistArgs = parseArgs sdistCmd updateCfg (False, 0)
+  where updateCfg (snapshot, verbose) fl = case fl of
+            Snapshot        -> (True,     verbose)
+            Verbose n       -> (snapshot, n)
+            _               -> error $ "Unexpected flag!"
+
+testCmd :: Cmd a
+testCmd = Cmd {
+        cmdName        = "test",
+        cmdHelp        = "Run the test suite, if any (configure with UserHooks).",
+        cmdDescription = "",  -- This can be a multi-line description
+        cmdOptions     = [cmd_help,cmd_verbose],
+        cmdAction      = TestCmd
+        }
+
+parseTestArgs :: [String] -> [OptDescr a] -> IO (Int, [a], [String])
+parseTestArgs = parseNoArgs testCmd
 
 registerCmd :: Cmd a
 registerCmd = Cmd {
@@ -461,25 +603,13 @@ type RegisterFlags = (Bool, Bool, Int)
 
 parseRegisterArgs :: RegisterFlags -> [String] -> [OptDescr a] ->
                      IO (RegisterFlags, [a], [String])
-parseRegisterArgs cfg args customOpts =
-  case getCmdOpt registerCmd customOpts args of
-    (flags, _, []) | hasHelpFlag flags -> do
-      printCmdHelp registerCmd customOpts
-      exitWith ExitSuccess
-    (flags, args', []) ->
-      return (updateCfg flags cfg, unliftFlags flags, args')
-    (_, _, errs) -> do putStrLn "Errors: "
-                       mapM_ putStrLn errs
-                       exitWith (ExitFailure 1)
-  where updateCfg (fl:flags) (uFlag, genScriptFlag, verbose) = updateCfg flags $
-          case fl of
-            UserFlag        -> (True, genScriptFlag, verbose)
+parseRegisterArgs = parseArgs registerCmd updateCfg
+  where updateCfg (uFlag, genScriptFlag, verbose) fl = case fl of
+            UserFlag        -> (True,  genScriptFlag, verbose)
             GlobalFlag      -> (False, genScriptFlag, verbose)
-            Verbose n       -> (uFlag,genScriptFlag, n)
-            GenScriptFlag   -> (uFlag, True, verbose)
-            Lift _          -> (uFlag,genScriptFlag, verbose)
+            Verbose n       -> (uFlag, genScriptFlag, n)
+            GenScriptFlag   -> (uFlag, True,          verbose)
             _               -> error $ "Unexpected flag!"
-        updateCfg [] t = t
 
 unregisterCmd :: Cmd a
 unregisterCmd = Cmd {
@@ -506,23 +636,32 @@ parseUnregisterArgs = parseRegisterArgs
 -- and help.
 
 parseNoArgs :: (Cmd a) -> [String] -> [OptDescr a] -> IO (Int, [a], [String])
-parseNoArgs cmd args customOpts =
+parseNoArgs cmd = parseArgs cmd updateCfg 0
+  where
+    updateCfg _ (Verbose n) = n
+    updateCfg _ _           = error "Unexpected flag!"
+
+-- |Helper function for commands with more options.
+
+parseArgs :: Cmd a -> (cfg -> Flag a -> cfg) -> cfg ->
+        [String] -> [OptDescr a] -> IO (cfg, [a], [String])
+parseArgs cmd updateCfg cfg args customOpts =
   case getCmdOpt cmd customOpts args of
     (flags, _, []) | hasHelpFlag flags -> do
       printCmdHelp cmd customOpts
       exitWith ExitSuccess
     (flags, args', []) ->
-      return (updateCmd flags 0, unliftFlags flags, args')
-    (_, _, errs) -> do putStrLn "Errors: "
-                       mapM_ putStrLn errs
-                       exitWith (ExitFailure 1)
+      let flags' = filter (not.isLift) flags in
+      return (foldl updateCfg cfg flags', unliftFlags flags, args')
+    (_, _, errs) -> putErrors errs
   where
-    updateCmd (fl:flags) _ = updateCmd flags $
-          case fl of
-            Verbose n -> n
-            _         -> error $ "Unexpected flag!"
-    updateCmd [] t = t
+    isLift (Lift _) = True
+    isLift _        = False
+    unliftFlags :: [Flag a] -> [a]
+    unliftFlags flags = [ fl | Lift fl <- flags ]
 
+putErrors :: [String] -> IO a
+putErrors errs = die $ "Errors:" ++ concat ['\n':err | err <- errs]
 
 
 #ifdef DEBUG
