@@ -5,7 +5,7 @@
 -- 
 -- Maintainer  :  Isaac Jones <ijones@syntaxpolice.org>
 -- Stability   :  alpha
--- Portability :  GHC
+-- Portability :  portable
 --
 -- Explanation: Perform the \"@.\/setup install@\" action.  Move files into
 -- place based on the prefix argument.
@@ -42,8 +42,6 @@ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. -}
 
 module Distribution.Simple.Install (
 	install,
-	mkBinDir,
-	mkLibDir,
 	hugsPackageDir,
 	hugsProgramsDirs,
 	hugsMainFilename,
@@ -52,7 +50,7 @@ module Distribution.Simple.Install (
 #endif
   ) where
 
-#if __GLASGOW_HASKELL__
+#if __GLASGOW_HASKELL__ && __GLASGOW_HASKELL__ < 604
 #if __GLASGOW_HASKELL__ < 603
 #include "config.h"
 #else
@@ -63,16 +61,24 @@ module Distribution.Simple.Install (
 import Distribution.PackageDescription (
 	PackageDescription(..), BuildInfo(..), Executable(..), Library (..),
 	setupMessage, hasLibs, withLib, libModules, withExe,
-	hcOptions)
+	hcOptions, autogenModuleName)
 import Distribution.Package (showPackageId, PackageIdentifier(pkgName))
-import Distribution.Simple.LocalBuildInfo(LocalBuildInfo(..))
-import Distribution.Simple.Utils(smartCopySources, copyFileVerbose, mkLibName, die)
-import Distribution.Setup (CompilerFlavor(..), Compiler(..))
+import Distribution.Program(ProgramConfiguration, Program(..), ProgramLocation(..),
+                            rawSystemProgram, ranlibProgram,
+                            lookupProgram, arProgram)
+import Distribution.Simple.LocalBuildInfo (
+        LocalBuildInfo(..), mkLibDir, mkBinDir, mkDataDir, mkProgDir)
+import Distribution.Simple.Utils(smartCopySources, copyFileVerbose, mkLibName,
+                                 mkProfLibName, mkGHCiLibName, die, rawSystemVerbose)
+import Distribution.Compiler (CompilerFlavor(..), Compiler(..))
+import Distribution.Setup (CopyFlags, CopyDest(..))
 
 import Control.Monad(when)
+import Data.List(any)
 import Data.Maybe(fromMaybe)
-import Distribution.Compat.Directory(createDirectoryIfMissing,removeDirectoryRecursive)
-import Distribution.Compat.FilePath(joinFileName, dllExtension, exeExtension,
+import Distribution.Compat.Directory(createDirectoryIfMissing, removeDirectoryRecursive,
+                                     findExecutable)
+import Distribution.Compat.FilePath(splitFileName,joinFileName, dllExtension, exeExtension,
 				    splitFileExt, joinFileExt)
 import System.IO.Error(try)
 import System.Directory(Permissions(..), getPermissions, setPermissions)
@@ -84,18 +90,27 @@ import HUnit (Test)
 -- |FIX: nhc isn't implemented yet.
 install :: PackageDescription
         -> LocalBuildInfo
-        -> (Maybe FilePath,Int) -- ^install-prefix, verbose
+        -> CopyFlags
         -> IO ()
-install pkg_descr lbi (install_prefixM,verbose) = do
+install pkg_descr lbi (copydest, verbose) = do
+  when (not (null (dataFiles pkg_descr))) $ do
+    let dataPref = mkDataDir pkg_descr lbi copydest
+    createDirectoryIfMissing True dataPref
+    flip mapM_ (dataFiles pkg_descr) $ \ file -> do
+      let (dir, _) = splitFileName file
+      createDirectoryIfMissing True (dataPref `joinFileName` dir)
+      copyFileVerbose verbose file (dataPref `joinFileName` file)
   let buildPref = buildDir lbi
-  let libPref = mkLibDir pkg_descr lbi install_prefixM
-  let targetLibPref = mkLibDir pkg_descr lbi Nothing
-  let binPref = mkBinDir pkg_descr lbi install_prefixM
+  let libPref = mkLibDir pkg_descr lbi copydest
+  let binPref = mkBinDir pkg_descr lbi copydest
   setupMessage ("Installing: " ++ libPref ++ " & " ++ binPref) pkg_descr
   case compilerFlavor (compiler lbi) of
-     GHC  -> do when (hasLibs pkg_descr) (installLibGHC verbose libPref buildPref pkg_descr)
+     GHC  -> do when (hasLibs pkg_descr) (installLibGHC verbose (withPrograms lbi) (withProfLib lbi) (withGHCiLib lbi) libPref buildPref pkg_descr)
                 installExeGhc verbose binPref buildPref pkg_descr
-     Hugs -> installHugs verbose libPref binPref targetLibPref buildPref pkg_descr
+     Hugs -> do
+       let progPref = mkProgDir pkg_descr lbi copydest
+       let targetProgPref = mkProgDir pkg_descr lbi NoCopyDest
+       installHugs verbose libPref progPref binPref targetProgPref buildPref pkg_descr
      _    -> die ("only installing with GHC or Hugs is implemented")
   return ()
   -- register step should be performed by caller.
@@ -109,20 +124,55 @@ installExeGhc verbose pref buildPref pkg_descr
     = do createDirectoryIfMissing True pref
          withExe pkg_descr $ \ (Executable e _ b) -> do
              let exeName = e `joinFileExt` exeExtension
-             copyFileVerbose verbose (buildPref `joinFileName` exeName) (pref `joinFileName` exeName)
+             copyFileVerbose verbose (buildPref `joinFileName` e `joinFileName` exeName) (pref `joinFileName` exeName)
 
--- |Install for ghc, .hi and .a
+-- |Install for ghc, .hi, .a and, if --with-ghci given, .o
 installLibGHC :: Int      -- ^verbose
+              -> ProgramConfiguration
+              -> Bool     -- ^has profiling library
+	      -> Bool     -- ^has GHCi libs
               -> FilePath -- ^install location
               -> FilePath -- ^Build location
               -> PackageDescription -> IO ()
-installLibGHC verbose pref buildPref pd@PackageDescription{library=Just l,
-                                                   package=p}
-    = do smartCopySources verbose (buildPref `joinFileName` (hsSourceDir $ libBuildInfo l)) pref (libModules pd) ["hi"] True
-         copyFileVerbose verbose (mkLibName buildPref (showPackageId p)) (mkLibName pref (showPackageId p))
-installLibGHC _ _ _ PackageDescription{library=Nothing}
+installLibGHC verbose programConf hasProf hasGHCi pref buildPref
+              pd@PackageDescription{library=Just l,
+                                    package=p}
+    = do smartCopySources verbose [buildPref] pref (libModules pd) ["hi"] True False
+         ifProf $ smartCopySources verbose [buildPref] pref (libModules pd) ["p_hi"] True False
+         let libTargetLoc = mkLibName pref (showPackageId p)
+             profLibTargetLoc = mkProfLibName pref (showPackageId p)
+	     libGHCiTargetLoc = mkGHCiLibName pref (showPackageId p)
+         copyFileVerbose verbose (mkLibName buildPref (showPackageId p)) libTargetLoc
+         ifProf $ copyFileVerbose verbose (mkProfLibName buildPref (showPackageId p)) profLibTargetLoc
+	 ifGHCi $ copyFileVerbose verbose (mkGHCiLibName buildPref (showPackageId p)) libGHCiTargetLoc
+
+         -- use ranlib or ar -s to build an index. this is necessary
+         -- on some systems like MacOS X.  If we can't find those,
+         -- don't worry too much about it.
+         let progName = programName $ ranlibProgram
+         mProg <- lookupProgram progName programConf
+         case foundProg mProg of
+           Just rl  -> do rawSystemProgram verbose rl [libTargetLoc]
+                          ifProf $ rawSystemProgram verbose rl [profLibTargetLoc]
+
+           Nothing -> do let progName = programName $ arProgram
+                         mProg <- lookupProgram progName programConf
+                         case mProg of
+                          Just ar  -> do rawSystemProgram verbose ar ["-s", libTargetLoc]
+                                         ifProf $ rawSystemProgram verbose ar ["-s", profLibTargetLoc]
+                          Nothing -> setupMessage  "Warning: Unable to generate index for library (missing ranlib and ar)" pd
+         return ()
+    where ifProf action = when hasProf (action >> return ())
+	  ifGHCi action = when hasGHCi (action >> return ())
+installLibGHC _ _ _ _ _ _ PackageDescription{library=Nothing}
     = die $ "Internal Error. installLibGHC called with no library."
 
+-- Also checks whether the program was actually found.
+foundProg :: Maybe Program -> Maybe Program
+foundProg Nothing = Nothing
+foundProg (Just Program{programLocation=EmptyLocation}) = Nothing
+foundProg x = x
+
 -- Install for Hugs
 -- For install, copy-prefix = prefix, but for copy they're different.
 -- The library goes in <copy-prefix>/lib/hugs/packages/<pkgname>
@@ -134,43 +184,45 @@ installLibGHC _ _ _ PackageDescription{library=Nothing}
 installHugs
     :: Int      -- ^verbose
     -> FilePath -- ^Library install location
+    -> FilePath -- ^Program install location
     -> FilePath -- ^Executable install location
-    -> FilePath -- ^Library location on target system
+    -> FilePath -- ^Program location on target system
     -> FilePath -- ^Build location
     -> PackageDescription
     -> IO ()
-installHugs verbose libPref binPref targetLibPref buildPref pkg_descr = do
+installHugs verbose libDir installProgDir binDir targetProgDir buildPref pkg_descr = do
     let pkg_name = pkgName (package pkg_descr)
     withLib pkg_descr () $ \ libInfo -> do
-        let pkgDir = libPref `joinFileName` "packages"
-                    `joinFileName` pkg_name
-        try $ removeDirectoryRecursive pkgDir
-        smartCopySources verbose buildPref pkgDir (libModules pkg_descr) hugsInstallSuffixes True
-    let progBuildDir = buildPref `joinFileName` "programs"
-    let progInstallDir = libPref `joinFileName` "programs"
-    let progTargetDir = targetLibPref `joinFileName` "programs"
-    when (not (null (executables pkg_descr))) $
-        createDirectoryIfMissing True binPref
+        try $ removeDirectoryRecursive libDir
+        smartCopySources verbose [buildPref] libDir (libModules pkg_descr) hugsInstallSuffixes True False
+    let buildProgDir = buildPref `joinFileName` "programs"
+    when (any (buildable . buildInfo) (executables pkg_descr)) $
+        createDirectoryIfMissing True binDir
     withExe pkg_descr $ \ exe -> do
-        let buildDir = progBuildDir `joinFileName` exeName exe
-        let installDir = progInstallDir `joinFileName` exeName exe
-        let targetDir = progTargetDir `joinFileName` exeName exe
+        let buildDir = buildProgDir `joinFileName` exeName exe
+        let installDir = installProgDir `joinFileName` exeName exe
+        let targetDir = targetProgDir `joinFileName` exeName exe
         try $ removeDirectoryRecursive installDir
-        smartCopySources verbose buildDir installDir
-            ("Main" : otherModules (buildInfo exe)) hugsInstallSuffixes True
-#ifndef mingw32_TARGET_OS
-        -- FIX (HUGS): works for Unix only
-        let targetName = targetDir `joinFileName` hugsMainFilename exe
-        let exeFile = binPref `joinFileName` exeName exe
+        smartCopySources verbose [buildDir] installDir
+            ("Main" : autogenModuleName pkg_descr : otherModules (buildInfo exe)) hugsInstallSuffixes True False
+        let targetName = "\"" ++ (targetDir `joinFileName` hugsMainFilename exe) ++ "\""
         -- FIX (HUGS): use extensions, and options from file too?
+        -- see http://hackage.haskell.org/trac/hackage/ticket/43
         let hugsOptions = hcOptions Hugs (options (buildInfo exe))
+#if mingw32_HOST_OS || mingw32_TARGET_OS
+        let exeFile = binDir `joinFileName` exeName exe `joinFileExt` "bat"
         let script = unlines [
-                "#! /bin/sh", 
+                "@echo off",
+                unwords ("runhugs" : hugsOptions ++ [targetName, "%*"])]
+#else
+        let exeFile = binDir `joinFileName` exeName exe
+        let script = unlines [
+                "#! /bin/sh",
                 unwords ("runhugs" : hugsOptions ++ [targetName, "\"$@\""])]
+#endif
         writeFile exeFile script
         perms <- getPermissions exeFile
         setPermissions exeFile perms { executable = True, readable = True }
-#endif
 
 hugsInstallSuffixes :: [String]
 hugsInstallSuffixes = ["hs", "lhs", dllExtension]
@@ -178,7 +230,7 @@ hugsInstallSuffixes = ["hs", "lhs", dllExtension]
 -- |Hugs library directory for a package
 hugsPackageDir :: PackageDescription -> LocalBuildInfo -> FilePath
 hugsPackageDir pkg_descr lbi =
-    mkLibDir pkg_descr lbi Nothing
+    mkLibDir pkg_descr lbi NoCopyDest
 	`joinFileName` "packages" `joinFileName` pkgName (package pkg_descr)
 
 -- |Hugs program directories for a package
@@ -186,7 +238,7 @@ hugsProgramsDirs :: PackageDescription -> LocalBuildInfo -> [FilePath]
 hugsProgramsDirs pkg_descr lbi =
     [exeDir `joinFileName` exeName exe |
          exe <- executables pkg_descr, buildable (buildInfo exe)]
-  where exeDir = mkLibDir pkg_descr lbi Nothing `joinFileName` "programs"
+  where exeDir = mkLibDir pkg_descr lbi NoCopyDest `joinFileName` "programs"
 
 -- |Filename used by Hugs for the main module of an executable.
 -- This is a simple filename, so that Hugs will look for any auxiliary
@@ -195,23 +247,6 @@ hugsMainFilename :: Executable -> FilePath
 hugsMainFilename exe = "Main" `joinFileExt` ext
   where (_, ext) = splitFileExt (modulePath exe)
 
--- -----------------------------------------------------------------------------
--- Installation policies
-
-mkLibDir :: PackageDescription -> LocalBuildInfo -> Maybe FilePath -> FilePath
-mkLibDir pkg_descr lbi install_prefixM = 
-  case compilerFlavor (compiler lbi) of
-    Hugs -> libDir `joinFileName` "hugs"
-    _ -> libDir `joinFileName` showPackageId (package pkg_descr)
-  where libDir = (fromMaybe (prefix lbi) install_prefixM)
-#ifndef mingw32_TARGET_OS
-                 `joinFileName` "lib"
-#endif
-
-mkBinDir :: PackageDescription -> LocalBuildInfo -> Maybe FilePath -> FilePath
-mkBinDir _ lbi install_prefixM = 
-  (fromMaybe (prefix lbi) install_prefixM) `joinFileName` "bin"
-
 -- ------------------------------------------------------------
 -- * Testing
 -- ------------------------------------------------------------
