@@ -5,7 +5,7 @@
 -- 
 -- Maintainer  :  Isaac Jones <ijones@syntaxpolice.org>
 -- Stability   :  alpha
--- Portability :  GHC, Hugs
+-- Portability :  portable
 --
 -- Explanation: Perform the \"@.\/setup register@\" action.
 -- Uses a drop-file for HC-PKG.  See also "Distribution.InstalledPackageInfo".
@@ -54,7 +54,7 @@ module Distribution.Simple.Register (
 #endif
   ) where
 
-#if __GLASGOW_HASKELL__
+#if __GLASGOW_HASKELL__ && __GLASGOW_HASKELL__ < 604
 #if __GLASGOW_HASKELL__ < 603
 #include "config.h"
 #else
@@ -62,9 +62,9 @@ module Distribution.Simple.Register (
 #endif
 #endif
 
-import Distribution.Simple.LocalBuildInfo (LocalBuildInfo(..))
-import Distribution.Simple.Install (mkLibDir)
-import Distribution.Setup (CompilerFlavor(..), Compiler(..), RegisterFlags)
+import Distribution.Simple.LocalBuildInfo (LocalBuildInfo(..), mkLibDir)
+import Distribution.Compiler (CompilerFlavor(..), Compiler(..))
+import Distribution.Setup (RegisterFlags, CopyDest(..))
 import Distribution.PackageDescription (setupMessage, PackageDescription(..),
 					BuildInfo(..), Library(..))
 import Distribution.Package (PackageIdentifier(..), showPackageId)
@@ -78,9 +78,10 @@ import Distribution.Simple.Install (hugsPackageDir)
 import Distribution.Simple.GHCPackageConfig (mkGHCPackageConfig, showGHCPackageConfig)
 import qualified Distribution.Simple.GHCPackageConfig
     as GHC (localPackageConfig, canWriteLocalPackageConfig, maybeCreateLocalPackageConfig)
-import Distribution.Compat.Directory (createDirectoryIfMissing,removeDirectoryRecursive,
-                                      setPermissions, getPermissions, executable
-                                     )
+import Distribution.Compat.Directory
+       (createDirectoryIfMissing,removeDirectoryRecursive,
+        setPermissions, getPermissions, Permissions(executable)
+       )
 import Distribution.Compat.FilePath (joinFileName)
 
 import System.Directory(doesFileExist, removeFile)
@@ -94,14 +95,14 @@ import HUnit (Test)
 #endif
 
 regScriptLocation :: FilePath
-#ifdef mingw32_TARGET_OS
+#if mingw32_HOST_OS || mingw32_TARGET_OS
 regScriptLocation = "register.bat"
 #else
 regScriptLocation = "register.sh"
 #endif
 
 unregScriptLocation :: FilePath
-#ifdef mingw32_TARGET_OS
+#if mingw32_HOST_OS || mingw32_TARGET_OS
 unregScriptLocation = "unregister.bat"
 #else
 unregScriptLocation = "unregister.sh"
@@ -151,23 +152,22 @@ register pkg_descr lbi (userInst, genScript, verbose)
 
 	let register_flags 
 		| ghc_63_plus = "update":
-#ifndef mingw32_TARGET_OS
+#if !(mingw32_HOST_OS || mingw32_TARGET_OS)
 		                 if genScript
                                     then []
                                     else 
 #endif
                                       [installedPkgConfigFile]
 		| otherwise   = "--update-package":
-#ifndef mingw32_TARGET_OS
+#if !(mingw32_HOST_OS || mingw32_TARGET_OS)
 				 if genScript
                                     then []
                                     else
 #endif
                                       ["--input-file="++installedPkgConfigFile]
         
-	let allFlags = "--auto-ghci-libs":
-		       (register_flags
-                        ++ config_flags)
+	let allFlags = register_flags
+                       ++ config_flags
                        ++ if ghc_63_plus && genScript then ["-"] else []
         let pkgTool = compilerPkgTool (compiler lbi)
 
@@ -177,7 +177,6 @@ register pkg_descr lbi (userInst, genScript, verbose)
                            pkgTool allFlags
          else rawSystemExit verbose pkgTool allFlags
 
-      -- FIX (HUGS):
       Hugs -> do
 	createDirectoryIfMissing True (hugsPackageDir pkg_descr lbi)
 	copyFileVerbose verbose installedPkgConfigFile
@@ -236,8 +235,8 @@ mkInstalledPackageInfo pkg_descr lbi
         IPI.exposed           = True,
 	IPI.exposedModules    = exposedModules lib,
 	IPI.hiddenModules     = otherModules bi,
-        IPI.importDirs        = [mkLibDir pkg_descr lbi Nothing],
-        IPI.libraryDirs       = (mkLibDir pkg_descr lbi Nothing) : extraLibDirs bi,
+        IPI.importDirs        = [mkLibDir pkg_descr lbi NoCopyDest],
+        IPI.libraryDirs       = (mkLibDir pkg_descr lbi NoCopyDest) : extraLibDirs bi,
         IPI.hsLibraries       = ["HS" ++ showPackageId (package pkg_descr)],
         IPI.extraLibraries    = extraLibs bi,
         IPI.includeDirs       = includeDirs bi,
@@ -259,7 +258,6 @@ unregister :: PackageDescription -> LocalBuildInfo -> RegisterFlags -> IO ()
 unregister pkg_descr lbi (user_unreg, genScript, verbose) = do
   setupMessage "Unregistering" pkg_descr
   let ghc_63_plus = compilerVersion (compiler lbi) >= Version [6,3] []
-  let theName = pkgName (package pkg_descr)
   case compilerFlavor (compiler lbi) of
     GHC -> do
 	config_flags <-
@@ -273,8 +271,8 @@ unregister pkg_descr lbi (user_unreg, genScript, verbose) = do
 			  return ["--config-file=" ++ localConf]
 		else return []
         let removeCmd = if ghc_63_plus
-                        then ["unregister",theName]
-                        else ["--remove-package="++theName]
+                        then ["unregister",showPackageId (package pkg_descr)]
+                        else ["--remove-package="++(pkgName $ package pkg_descr)]
 	rawSystemEmit unregScriptLocation genScript verbose (compilerPkgTool (compiler lbi))
 	    (removeCmd++config_flags)
     Hugs -> do
@@ -294,7 +292,7 @@ rawSystemEmit :: FilePath -- ^Script name
 rawSystemEmit _ False verbosity path args
     = rawSystemExit verbosity path args
 rawSystemEmit scriptName True verbosity path args = do
-#ifdef mingw32_TARGET_OS
+#if mingw32_HOST_OS || mingw32_TARGET_OS
   writeFile scriptName ("@" ++ path ++ concatMap (' ':) args)
 #else
   writeFile scriptName ("#!/bin/sh\n\n"
@@ -312,7 +310,7 @@ rawSystemPipe :: FilePath -- ^Script location
               -> [String] -- ^Args
               -> IO ()
 rawSystemPipe scriptName verbose pipeFrom path args = do
-#ifdef mingw32_TARGET_OS
+#if mingw32_HOST_OS || mingw32_TARGET_OS
   writeFile scriptName ("@" ++ path ++ concatMap (' ':) args)
 #else
   writeFile scriptName ("#!/bin/sh\n\n"
