@@ -5,7 +5,7 @@
 -- 
 -- Maintainer  :  Isaac Jones <ijones@syntaxpolice.org>
 -- Stability   :  alpha
--- Portability :  
+-- Portability :  portable
 --
 
 {- Copyright (c) 2003-2004, Simon Marlow
@@ -52,15 +52,19 @@ module Distribution.Simple.SrcDist (
 import Distribution.PackageDescription
 	(PackageDescription(..), BuildInfo(..), Executable(..), Library(..),
          setupMessage, libModules)
-import Distribution.Package (showPackageId)
+import Distribution.Package (showPackageId, PackageIdentifier(pkgVersion))
+import Distribution.Version (Version(versionBranch))
 import Distribution.Simple.Utils
-        (smartCopySources, die, findPackageDesc, copyFileVerbose)
+        (smartCopySources, die, findPackageDesc, findFile, copyFileVerbose)
 import Distribution.PreProcess (PPSuffixHandler, ppSuffixes, removePreprocessed)
 
 import Control.Monad(when)
+import Data.Char (isSpace, toLower)
+import Data.List (isPrefixOf)
 import System.Cmd (system)
+import System.Time (getClockTime, toCalendarTime, CalendarTime(..))
 import Distribution.Compat.Directory (doesFileExist, doesDirectoryExist,
-         getCurrentDirectory, createDirectoryIfMissing)
+         getCurrentDirectory, createDirectoryIfMissing, removeDirectoryRecursive)
 import Distribution.Compat.FilePath (joinFileName, splitFileName)
 
 #ifdef DEBUG
@@ -72,23 +76,39 @@ import HUnit (Test)
 sdist :: FilePath -- ^build prefix (temp dir)
       -> FilePath -- ^TargetPrefix
       -> Int      -- ^verbose
+      -> Bool     -- ^snapshot
       -> [PPSuffixHandler]  -- ^ extra preprocessors (includes suffixes)
       -> PackageDescription
       -> IO ()
-sdist tmpDir targetPref verbose pps pkg_descr = do
+sdist tmpDir targetPref verbose snapshot pps pkg_descr_orig = do
+  time <- getClockTime
+  ct <- toCalendarTime time
+  let date = ctYear ct*10000 + (fromEnum (ctMonth ct) + 1)*100 + ctDay ct
+  let pkg_descr
+        | snapshot  = updatePackage (updatePkgVersion
+                        (updateVersionBranch (++ [date]))) pkg_descr_orig
+        | otherwise = pkg_descr_orig
   setupMessage "Building source dist for" pkg_descr
   ex <- doesDirectoryExist tmpDir
   when ex (die $ "Source distribution already in place. please move: " ++ tmpDir)
   let targetDir = tmpDir `joinFileName` (nameVersion pkg_descr)
+  createDirectoryIfMissing True targetDir
   -- maybe move the library files into place
   maybe (return ()) (\l -> prepareDir verbose targetDir pps (libModules pkg_descr) (libBuildInfo l))
                     (library pkg_descr)
   -- move the executables into place
   flip mapM_ (executables pkg_descr) $ \ (Executable _ mainPath exeBi) -> do
     prepareDir verbose targetDir pps [] exeBi
-    copyFileTo verbose targetDir (hsSourceDir exeBi `joinFileName` mainPath)
+    srcMainFile <- findFile (hsSourceDirs exeBi) mainPath
+    copyFileTo verbose targetDir srcMainFile
+  flip mapM_ (dataFiles pkg_descr) $ \ file -> do
+    let (dir, _) = splitFileName file
+    createDirectoryIfMissing True (targetDir `joinFileName` dir)
+    copyFileVerbose verbose file (targetDir `joinFileName` file)
   when (not (null (licenseFile pkg_descr))) $
     copyFileTo verbose targetDir (licenseFile pkg_descr)
+  flip mapM_ (extraSrcFiles pkg_descr) $ \ fpath -> do
+    copyFileTo verbose targetDir fpath
   -- setup isn't listed in the description file.
   hsExists <- doesFileExist "Setup.hs"
   lhsExists <- doesFileExist "Setup.lhs"
@@ -99,13 +119,35 @@ sdist tmpDir targetPref verbose pps pkg_descr = do
                 "main = defaultMainWithHooks defaultUserHooks"]
   -- the description file itself
   descFile <- getCurrentDirectory >>= findPackageDesc
-  copyFileTo verbose targetDir descFile
+  let targetDescFile = targetDir `joinFileName` descFile
+  -- We could just writePackageDescription targetDescFile pkg_descr,
+  -- but that would lose comments and formatting.
+  if snapshot then do
+      contents <- readFile descFile
+      writeFile targetDescFile $
+          unlines $ map (appendVersion date) $ lines $ contents
+    else copyFileVerbose verbose descFile targetDescFile
 
+  let tarBallFilePath = targetPref `joinFileName` tarBallName pkg_descr
   system $ "(cd " ++ tmpDir
            ++ ";tar cf - " ++ (nameVersion pkg_descr) ++ ") | gzip -9 >"
-           ++ (targetPref `joinFileName` (tarBallName pkg_descr))
-  system $ "rm -rf " ++ tmpDir
-  putStrLn "Source tarball created."
+           ++ tarBallFilePath
+  removeDirectoryRecursive tmpDir
+  putStrLn $ "Source tarball created: " ++ tarBallFilePath
+
+  where
+    updatePackage f pd = pd { package = f (package pd) }
+    updatePkgVersion f pkg = pkg { pkgVersion = f (pkgVersion pkg) }
+    updateVersionBranch f v = v { versionBranch = f (versionBranch v) }
+
+    appendVersion :: Int -> String -> String
+    appendVersion n line
+      | "version:" `isPrefixOf` map toLower line =
+            trimTrailingSpace line ++ "." ++ show n
+      | otherwise = line
+
+    trimTrailingSpace :: String -> String
+    trimTrailingSpace = reverse . dropWhile isSpace . reverse
 
 -- |Move the sources into place based on buildInfo
 prepareDir :: Int       -- ^verbose
@@ -114,11 +156,10 @@ prepareDir :: Int       -- ^verbose
            -> [String]  -- ^Exposed modules
            -> BuildInfo
            -> IO ()
-prepareDir verbose inPref pps mods BuildInfo{hsSourceDir=srcDir, otherModules=mods', cSources=cfiles}
-    = do let pref = inPref `joinFileName` srcDir
-         let suff = ppSuffixes pps  ++ ["hs", "lhs"]
-         smartCopySources verbose srcDir pref (mods++mods') suff True
-         removePreprocessed pref mods suff
+prepareDir verbose inPref pps mods BuildInfo{hsSourceDirs=srcDirs, otherModules=mods', cSources=cfiles}
+    = do let suff = ppSuffixes pps  ++ ["hs", "lhs"]
+         smartCopySources verbose srcDirs inPref (mods++mods') suff True True
+         removePreprocessed (map (joinFileName inPref) srcDirs) mods suff
          mapM_ (copyFileTo verbose inPref) cfiles
 
 copyFileTo :: Int -> FilePath -> FilePath -> IO ()
@@ -131,7 +172,7 @@ copyFileTo verbose dir file = do
 
 -- |The file name of the tarball
 tarBallName :: PackageDescription -> FilePath
-tarBallName p = (nameVersion p) ++ ".tgz"
+tarBallName p = (nameVersion p) ++ ".tar.gz"
 
 nameVersion :: PackageDescription -> String
 nameVersion = showPackageId . package
