@@ -6,7 +6,7 @@
 -- 
 -- Maintainer  :  Isaac Jones <ijones@syntaxpolice.org>
 -- Stability   :  alpha
--- Portability :  GHC
+-- Portability :  portable
 --
 -- Explanation: Misc. Utilities, especially file-related utilities.
 -- Stuff used by multiple modules that doesn't fit elsewhere.
@@ -43,6 +43,8 @@ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. -}
 
 module Distribution.Simple.Utils (
 	die,
+	dieWithLocation,
+	warn,
 	rawSystemPath,
         rawSystemVerbose,
 	rawSystemExit,
@@ -53,11 +55,12 @@ module Distribution.Simple.Utils (
         copyFileVerbose,
         moduleToFilePath,
         mkLibName,
+        mkProfLibName,
+	mkGHCiLibName,
         currentDir,
         dotToSep,
 	withTempFile,
-	getOptionsFromSource,
-	stripComments,
+	findFile,
         defaultPackageDesc,
         findPackageDesc,
 	defaultHookedPackageDesc,
@@ -67,7 +70,7 @@ module Distribution.Simple.Utils (
 #endif
   ) where
 
-#if __GLASGOW_HASKELL__
+#if __GLASGOW_HASKELL__ && __GLASGOW_HASKELL__ < 604
 #if __GLASGOW_HASKELL__ < 603
 #include "config.h"
 #else
@@ -77,27 +80,23 @@ module Distribution.Simple.Utils (
 
 import Distribution.Compat.RawSystem (rawSystem)
 import Distribution.Compat.Exception (finally)
-import Distribution.Compat.FilePath (dropPrefix)
-import Distribution.Extension (Extension)
-import Distribution.Setup (CompilerFlavor(..))
-import Distribution.PreProcess.Unlit (unlit)
 
-import Control.Monad(when)
-import Data.Char(isSpace)
-import Data.List(nub, isSuffixOf)
-import Data.Maybe(mapMaybe)
-import System.IO (hPutStr, stderr, hFlush, stdout)
+import Control.Monad(when, filterM)
+import Data.List (nub)
+import System.Environment (getProgName)
+import System.IO (hPutStrLn, stderr, hFlush, stdout)
 import System.IO.Error
 import System.Exit
-#if (__GLASGOW_HASKELL__ || __HUGS__) && !defined(mingw32_TARGET_OS)
+#if (__GLASGOW_HASKELL__ || __HUGS__) && !(mingw32_HOST_OS || mingw32_TARGET_OS)
 import System.Posix.Internals (c_getpid)
 #endif
 
 import Distribution.Compat.FilePath
 	(splitFileName, splitFileExt, joinFileName, joinFileExt,
 	pathSeparator)
-import System.Directory (getDirectoryContents, getCurrentDirectory,
-                        doesFileExist, removeFile)
+import System.Directory (getDirectoryContents, getCurrentDirectory
+			, doesFileExist, removeFile, getPermissions
+			, Permissions(executable))
 
 import Distribution.Compat.Directory (copyFile,findExecutable,createDirectoryIfMissing)
 
@@ -105,14 +104,24 @@ import Distribution.Compat.Directory (copyFile,findExecutable,createDirectoryIfM
 import HUnit ((~:), (~=?), Test(..), assertEqual)
 #endif
 
--- -----------------------------------------------------------------------------
--- Utils for setup
+-- ------------------------------------------------------------------------------- Utils for setup
+
+dieWithLocation :: FilePath -> (Maybe Int) -> String -> IO a
+dieWithLocation fname Nothing msg = die (fname ++ ": " ++ msg)
+dieWithLocation fname (Just n) msg = die (fname ++ ":" ++ show n ++ ": " ++ msg)
 
 die :: String -> IO a
-die msg = do hFlush stdout; hPutStr stderr (msg++"\n"); exitWith (ExitFailure 1)
+die msg = do
+  hFlush stdout
+  pname <- getProgName
+  hPutStrLn stderr (pname ++ ": " ++ msg)
+  exitWith (ExitFailure 1)
 
 warn :: String -> IO ()
-warn msg = do hFlush stdout; hPutStr stderr ("Warning: " ++ msg++"\n")
+warn msg = do
+  hFlush stdout
+  pname <- getProgName
+  hPutStrLn stderr (pname ++ ": Warning: " ++ msg)
 
 -- -----------------------------------------------------------------------------
 -- rawSystem variants
@@ -120,17 +129,20 @@ rawSystemPath :: Int -> String -> [String] -> IO ExitCode
 rawSystemPath verbose prog args = do
   r <- findExecutable prog
   case r of
-    Nothing -> die ("Cannot find: " ++ prog)
-    Just path -> do
-      when (verbose > 0) $
-        putStrLn (path ++ concatMap (' ':) args)
-      rawSystem path args
+    Nothing   -> die ("Cannot find: " ++ prog)
+    Just path -> rawSystemVerbose verbose path args
 
-rawSystemVerbose :: Int -> String -> [String] -> IO ExitCode
+rawSystemVerbose :: Int -> FilePath -> [String] -> IO ExitCode
 rawSystemVerbose verbose prog args = do
       when (verbose > 0) $
         putStrLn (prog ++ concatMap (' ':) args)
-      rawSystem prog args
+      e <- doesFileExist prog
+      if e
+         then do perms <- getPermissions prog
+                 if (executable perms)
+                    then rawSystem prog args
+                    else die ("Error: file is not executable: " ++ show prog)
+         else die ("Error: file does not exist: " ++ show prog)
 
 maybeExit :: IO ExitCode -> IO ()
 maybeExit cmd = do
@@ -168,13 +180,26 @@ moduleToFilePath :: [FilePath] -- ^search locations
                  -> IO [FilePath]
 
 moduleToFilePath pref s possibleSuffixes
-    = do let possiblePaths = concatMap (searchModuleToPossiblePaths s possibleSuffixes) pref
-         matchList <- mapM (\x -> do y <- doesFileExist x; return (x, y)) possiblePaths
-         return [x | (x, True) <- matchList]
+    = filterM doesFileExist $
+          concatMap (searchModuleToPossiblePaths s possibleSuffixes) pref
     where searchModuleToPossiblePaths :: String -> [String] -> FilePath -> [FilePath]
           searchModuleToPossiblePaths s' suffs searchP
               = moduleToPossiblePaths searchP s' suffs
 
+-- |Like 'moduleToFilePath', but return the location and the rest of
+-- the path as separate results.
+moduleToFilePath2
+    :: [FilePath] -- ^search locations
+    -> String   -- ^Module Name
+    -> [String] -- ^possible suffixes
+    -> IO [(FilePath, FilePath)] -- ^locations and relative names
+moduleToFilePath2 locs mname possibleSuffixes
+    = filterM exists $
+        [(loc, fname `joinFileExt` ext) | loc <- locs, ext <- possibleSuffixes]
+  where
+    fname = dotToSep mname
+    exists (loc, relname) = doesFileExist (loc `joinFileName` relname)
+
 -- |Get the possible file paths based on this module name.
 moduleToPossiblePaths :: FilePath -- ^search prefix
                       -> String -- ^module name
@@ -184,6 +209,17 @@ moduleToPossiblePaths searchPref s possibleSuffixes =
   let fname = searchPref `joinFileName` (dotToSep s)
   in [fname `joinFileExt` ext | ext <- possibleSuffixes]
 
+findFile :: [FilePath]    -- ^search locations
+         -> FilePath      -- ^File Name
+         -> IO FilePath
+findFile prefPathsIn locPath = do
+  let prefPaths = nub prefPathsIn -- ignore dups
+  paths <- filterM doesFileExist [prefPath `joinFileName` locPath | prefPath <- prefPaths]
+  case nub paths of -- also ignore dups, though above nub should fix this.
+    [path] -> return path
+    []     -> die (locPath ++ " doesn't exist")
+    paths  -> die (locPath ++ " is found in multiple places:" ++ unlines (map ((++) "    ") paths))
+
 dotToSep :: String -> String
 dotToSep = map dts
   where
@@ -196,33 +232,32 @@ dotToSep = map dts
 -- directory.
 
 smartCopySources :: Int      -- ^verbose
-            -> FilePath -- ^build prefix (location of objects)
+            -> [FilePath] -- ^build prefix (location of objects)
             -> FilePath -- ^Target directory
             -> [String] -- ^Modules
             -> [String] -- ^search suffixes
             -> Bool     -- ^Exit if no such modules
+            -> Bool     -- ^Preserve directory structure
             -> IO ()
-smartCopySources verbose pref targetDir sources searchSuffixes exitIfNone
+smartCopySources verbose srcDirs targetDir sources searchSuffixes exitIfNone preserveDirs
     = do createDirectoryIfMissing True targetDir
+         allLocations <- mapM moduleToFPErr sources
+         let copies = [(srcDir `joinFileName` name,
+                        if preserveDirs 
+                          then targetDir `joinFileName` srcDir `joinFileName` name
+                          else targetDir `joinFileName` name) |
+                       (srcDir, name) <- concat allLocations]
 	 -- Create parent directories for everything:
-         sourceLocs' <- mapM moduleToFPErr sources
-         let sourceLocs = concat $ filter (not . null) sourceLocs'
-         let sourceLocsNoPref -- get rid of the prefix, for target location.
-                 = if null pref || pref == currentDir then sourceLocs
-                   else map (dropPrefix pref) sourceLocs
-	 mapM (createDirectoryIfMissing True)
-		  $ nub [fst (splitFileName (targetDir `joinFileName` x))
-		   | x <- sourceLocsNoPref, fst (splitFileName x) /= "."]
+	 mapM_ (createDirectoryIfMissing True) $ nub $
+             [fst (splitFileName targetFile) | (_, targetFile) <- copies]
 	 -- Put sources into place:
-	 sequence_ [copyFileVerbose verbose x (targetDir `joinFileName` y)
-                      | (x,y) <- (zip sourceLocs sourceLocsNoPref)]
-	 return ()
+	 sequence_ [copyFileVerbose verbose srcFile destFile |
+                    (srcFile, destFile) <- copies]
     where moduleToFPErr m
-              = do p <- moduleToFilePath [pref] m searchSuffixes
+              = do p <- moduleToFilePath2 srcDirs m searchSuffixes
                    when (null p && exitIfNone)
-                            (putStrLn ("Error: Could not find module: " ++ m
-                                       ++ " with any suffix: " ++ (show searchSuffixes))
-                             >> exitWith (ExitFailure 1))
+                            (die ("Error: Could not find module: " ++ m
+                                       ++ " with any suffix: " ++ (show searchSuffixes)))
                    return p
 
 copyFileVerbose :: Int -> FilePath -> FilePath -> IO ()
@@ -241,6 +276,16 @@ mkLibName :: FilePath -- ^file Prefix
           -> String
 mkLibName pref lib = pref `joinFileName` ("libHS" ++ lib ++ ".a")
 
+mkProfLibName :: FilePath -- ^file Prefix
+              -> String   -- ^library name.
+              -> String
+mkProfLibName pref lib = mkLibName pref (lib++"_p")
+
+mkGHCiLibName :: FilePath -- ^file Prefix
+              -> String   -- ^library name.
+              -> String
+mkGHCiLibName pref lib = pref `joinFileName` ("HS" ++ lib ++ ".o")
+
 -- -----------------------------------------------------------------------------
 -- * temporary file names
 -- -----------------------------------------------------------------------------
@@ -259,7 +304,7 @@ withTempFile tmp_dir extn action
 	   if b then findTempName (x+1)
 		else action path `finally` try (removeFile path)
 
-#ifdef mingw32_TARGET_OS
+#if mingw32_HOST_OS || mingw32_TARGET_OS
 foreign import ccall unsafe "_getpid" getProcessID :: IO Int
 		 -- relies on Int == Int32 on Windows
 #elif __GLASGOW_HASKELL__ || __HUGS__
@@ -270,79 +315,6 @@ getProcessID = System.Posix.Internals.c_getpid >>= return . fromIntegral
 foreign import ccall unsafe "getpid" getProcessID :: IO Int
 #endif
 
--- ------------------------------------------------------------
--- * options in source files
--- ------------------------------------------------------------
-
--- |Read the initial part of a source file, before any Haskell code,
--- and return the contents of any LANGUAGE, OPTIONS and INCLUDE pragmas.
-getOptionsFromSource
-    :: FilePath
-    -> IO ([Extension],                 -- LANGUAGE pragma, if any
-           [(CompilerFlavor,[String])], -- OPTIONS_FOO pragmas
-           [String]                     -- INCLUDE pragmas
-          )
-getOptionsFromSource file = do
-    text <- readFile file
-    return $ foldr appendOptions ([],[],[]) $ map getOptions $
-	takeWhileJust $ map getPragma $
-	filter textLine $ map (dropWhile isSpace) $ lines $
-	stripComments True $
-	if ".lhs" `isSuffixOf` file then unlit file text else text
-  where textLine [] = False
-	textLine ('#':_) = False
-	textLine _ = True
-
-	getPragma :: String -> Maybe [String]
-	getPragma line = case words line of
-	    ("{-#" : rest) | last rest == "#-}" -> Just (init rest)
-	    _ -> Nothing
-
-	getOptions ("OPTIONS":opts) = ([], [(GHC, opts)], [])
-	getOptions ("OPTIONS_GHC":opts) = ([], [(GHC, opts)], [])
-	getOptions ("OPTIONS_NHC98":opts) = ([], [(NHC, opts)], [])
-	getOptions ("OPTIONS_HUGS":opts) = ([], [(Hugs, opts)], [])
-	getOptions ("LANGUAGE":ws) = (mapMaybe readExtension ws, [], [])
-	  where	readExtension :: String -> Maybe Extension
-		readExtension w = case reads w of
-		    [(ext, "")] -> Just ext
-		    [(ext, ",")] -> Just ext
-		    _ -> Nothing
-	getOptions ("INCLUDE":ws) = ([], [], ws)
-	getOptions _ = ([], [], [])
-
-	appendOptions (exts, opts, incs) (exts', opts', incs')
-          = (exts++exts', opts++opts', incs++incs')
-
--- takeWhileJust f = map fromJust . takeWhile isJust
-takeWhileJust :: [Maybe a] -> [a]
-takeWhileJust (Just x:xs) = x : takeWhileJust xs
-takeWhileJust _ = []
-
--- |Strip comments from Haskell source.
-stripComments
-    :: Bool	-- ^ preserve pragmas?
-    -> String	-- ^ input source text
-    -> String
-stripComments keepPragmas = stripCommentsLevel 0
-  where stripCommentsLevel :: Int -> String -> String
-	stripCommentsLevel 0 ('-':'-':cs) =	-- FIX: symbols like -->
-	    stripCommentsLevel 0 (dropWhile (/= '\n') cs)
-	stripCommentsLevel 0 ('{':'-':'#':cs)
-	  | keepPragmas = '{' : '-' : '#' : copyPragma cs
-	stripCommentsLevel n ('{':'-':cs) = stripCommentsLevel (n+1) cs
-	stripCommentsLevel 0 (c:cs) = c : stripCommentsLevel 0 cs
-	stripCommentsLevel n ('-':'}':cs) = stripCommentsLevel (n-1) cs
-	stripCommentsLevel n (c:cs) = stripCommentsLevel n cs
-	stripCommentsLevel _ [] = []
-
-	copyPragma ('#':'-':'}':cs) = '#' : '-' : '}' : stripCommentsLevel 0 cs
-	copyPragma (c:cs) = c : copyPragma cs
-	copyPragma [] = []
-
-
-
-
 -- ------------------------------------------------------------
 -- * Finding the description file
 -- ------------------------------------------------------------
@@ -427,9 +399,9 @@ hunitTests :: [Test]
 hunitTests
     = let suffixes = ["hs", "lhs"]
           in [TestCase $
-#ifdef mingw32_TARGET_OS
-       do mp1 <- moduleToFilePath "" "Distribution.Simple.Build" suffixes --exists
-          mp2 <- moduleToFilePath "" "Foo.Bar" suffixes    -- doesn't exist
+#if mingw32_HOST_OS || mingw32_TARGET_OS
+       do mp1 <- moduleToFilePath [""] "Distribution.Simple.Build" suffixes --exists
+          mp2 <- moduleToFilePath [""] "Foo.Bar" suffixes    -- doesn't exist
           assertEqual "existing not found failed"
                    (Just "Distribution\\Simple\\Build.hs") mp1
           assertEqual "not existing not nothing failed" Nothing mp2,
