@@ -62,9 +62,9 @@ module Control.Concurrent (
 #endif
 	-- $merge
 
+#ifdef __GLASGOW_HASKELL__
 	-- * Bound Threads
 	-- $boundthreads
-#ifdef __GLASGOW_HASKELL__
 	rtsSupportsBoundThreads,
 	forkOS,
 	isCurrentThreadBound,
@@ -91,11 +91,13 @@ import Prelude
 import Control.Exception as Exception
 
 #ifdef __GLASGOW_HASKELL__
-import GHC.Conc
+import GHC.Conc		( ThreadId(..), myThreadId, killThread, yield,
+			  threadDelay, threadWaitRead, threadWaitWrite,
+			  forkIO, childHandler )
 import GHC.TopHandler   ( reportStackOverflow, reportError )
 import GHC.IOBase	( IO(..) )
 import GHC.IOBase	( unsafeInterleaveIO )
-import GHC.IOBase   ( newIORef, readIORef, writeIORef )
+import GHC.IOBase	( newIORef, readIORef, writeIORef )
 import GHC.Base
 
 import Foreign.StablePtr
@@ -143,7 +145,7 @@ In GHC, threads may also communicate via exceptions.
 
     Scheduling may be either pre-emptive or co-operative,
     depending on the implementation of Concurrent Haskell (see below
-    for imformation related to specific compilers).  In a co-operative
+    for information related to specific compilers).  In a co-operative
     system, context switches only occur when you use one of the
     primitives defined in this module.  This means that programs such
     as:
@@ -177,7 +179,7 @@ functions blocks only the thread making the call.
 id2TSO :: ThreadId -> ThreadId#
 id2TSO (ThreadId t) = t
 
-foreign import ccall unsafe "cmp_thread" cmp_thread :: ThreadId# -> ThreadId# -> Int
+foreign import ccall unsafe "cmp_thread" cmp_thread :: ThreadId# -> ThreadId# -> CInt
 -- Returns -1, 0, 1
 
 cmpThread :: ThreadId -> ThreadId -> Ordering
@@ -203,35 +205,6 @@ instance Show ThreadId where
    	showString "ThreadId " . 
         showsPrec d (getThreadId (id2TSO t))
 
-{- |
-This sparks off a new thread to run the 'IO' computation passed as the
-first argument, and returns the 'ThreadId' of the newly created
-thread.
-
-The new thread will be a lightweight thread; if you want to use a foreign
-library that uses thread-local storage, use 'forkOS' instead.
--}
-forkIO :: IO () -> IO ThreadId
-forkIO action = IO $ \ s -> 
-   case (fork# action_plus s) of (# s1, id #) -> (# s1, ThreadId id #)
- where
-  action_plus = Exception.catch action childHandler
-
-childHandler :: Exception -> IO ()
-childHandler err = Exception.catch (real_handler err) childHandler
-
-real_handler :: Exception -> IO ()
-real_handler ex =
-  case ex of
-	-- ignore thread GC and killThread exceptions:
-	BlockedOnDeadMVar            -> return ()
-	AsyncException ThreadKilled  -> return ()
-
-	-- report all others:
-	AsyncException StackOverflow -> reportStackOverflow False
-	ErrorCall s -> reportError False s
-	other       -> reportError False (showsPrec 0 other "\n")
-
 #endif /* __GLASGOW_HASKELL__ */
 
 #ifndef __HUGS__
@@ -308,6 +281,7 @@ nmergeIO lss
     mapIO f xs = sequence (map f xs)
 #endif /* __HUGS__ */
 
+#ifdef __GLASGOW_HASKELL__
 -- ---------------------------------------------------------------------------
 -- Bound Threads
 
@@ -342,8 +316,6 @@ libraries (OpenGL, for example) will not work from a thread created using
 from @main@ or from a @foreign export@.
 -}
 
-#ifdef __GLASGOW_HASKELL__
-
 -- | 'True' if bound threads are supported.
 -- If @rtsSupportsBoundThreads@ is 'False', 'isCurrentThreadBound'
 -- will always return 'False' and both 'forkOS' and 'runInBoundThread' will
@@ -456,7 +428,7 @@ runInUnboundThread action = do
                 Right result -> return result
         else action
 	
-#endif
+#endif /* __GLASGOW_HASKELL__ */
 
 -- ---------------------------------------------------------------------------
 -- More docs
@@ -478,8 +450,8 @@ runInUnboundThread action = do
 
 >   myForkIO :: IO () -> IO (MVar ())
 >   myForkIO io = do
->     mvar \<- newEmptyMVar
->     forkIO (io \`finally\` putMVar mvar ())
+>     mvar <- newEmptyMVar
+>     forkIO (io `finally` putMVar mvar ())
 >     return mvar
 
       Note that we use 'finally' from the
@@ -490,25 +462,26 @@ runInUnboundThread action = do
       A better method is to keep a global list of all child
       threads which we should wait for at the end of the program:
 
->     children :: MVar [MVar ()]
->     children = unsafePerformIO (newMVar [])
->     
->     waitForChildren :: IO ()
->     waitForChildren = do
->     	(mvar:mvars) \<- takeMVar children
->     	putMVar children mvars
->     	takeMVar mvar
->     	waitForChildren
->     
->     forkChild :: IO () -> IO ()
->     forkChild io = do
->     	 mvar \<- newEmptyMVar
->     	 forkIO (p \`finally\` putMVar mvar ())
->     	 childs \<- takeMVar children
->     	 putMVar children (mvar:childs)
->     
->     later = flip finally
->     
+>    children :: MVar [MVar ()]
+>    children = unsafePerformIO (newMVar [])
+>    
+>    waitForChildren :: IO ()
+>    waitForChildren = do
+>      cs <- takeMVar children
+>      case cs of
+>        []   -> return ()
+>        m:ms -> do
+>    	    putMVar children ms
+>    	    takeMVar m
+>    	    waitForChildren
+>    
+>    forkChild :: IO () -> IO ()
+>    forkChild io = do
+>    	 mvar <- newEmptyMVar
+>    	 childs <- takeMVar children
+>    	 putMVar children (mvar:childs)
+>    	 forkIO (io `finally` putMVar mvar ())
+>
 >     main =
 >     	later waitForChildren $
 >     	...
@@ -528,7 +501,7 @@ runInUnboundThread action = do
       a thread may be pre-empted whenever it allocates some memory,
       which unfortunately means that tight loops which do no
       allocation tend to lock out other threads (this only seems to
-      happen with pathalogical benchmark-style code, however).
+      happen with pathological benchmark-style code, however).
 
       The rescheduling timer runs on a 20ms granularity by
       default, but this may be altered using the
